@@ -1,14 +1,14 @@
 from pathlib import Path
 
 import pytest
+from twisted.web.http import H2_ENABLED
 
 from scrapy.utils.reactor import install_reactor
-
 from tests.keys import generate_keys
 
 
 def _py_files(folder):
-    return (str(p) for p in Path(folder).rglob('*.py'))
+    return (str(p) for p in Path(folder).rglob("*.py"))
 
 
 collect_ignore = [
@@ -20,10 +20,19 @@ collect_ignore = [
     *_py_files("tests/CrawlerRunner"),
 ]
 
-for line in open('tests/ignores.txt'):
-    file_path = line.strip()
-    if file_path and file_path[0] != '#':
-        collect_ignore.append(file_path)
+with Path("tests/ignores.txt").open(encoding="utf-8") as reader:
+    for line in reader:
+        file_path = line.strip()
+        if file_path and file_path[0] != "#":
+            collect_ignore.append(file_path)
+
+if not H2_ENABLED:
+    collect_ignore.extend(
+        (
+            "scrapy/core/downloader/handlers/http2.py",
+            *_py_files("scrapy/core/http2"),
+        )
+    )
 
 
 @pytest.fixture()
@@ -32,16 +41,6 @@ def chdir(tmpdir):
     tmpdir.chdir()
 
 
-def pytest_collection_modifyitems(session, config, items):
-    # Avoid executing tests when executing `--flake8` flag (pytest-flake8)
-    try:
-        from pytest_flake8 import Flake8Item
-        if config.getoption('--flake8'):
-            items[:] = [item for item in items if isinstance(item, Flake8Item)]
-    except ImportError:
-        pass
-
-
 def pytest_addoption(parser):
     parser.addoption(
         "--reactor",
@@ -50,7 +49,7 @@ def pytest_addoption(parser):
     )
 
 
-@pytest.fixture(scope='class')
+@pytest.fixture(scope="class")
 def reactor_pytest(request):
     if not request.cls:
         # doctests
@@ -61,8 +60,17 @@ def reactor_pytest(request):
 
 @pytest.fixture(autouse=True)
 def only_asyncio(request, reactor_pytest):
-    if request.node.get_closest_marker('only_asyncio') and reactor_pytest != 'asyncio':
-        pytest.skip('This test is only run with --reactor=asyncio')
+    if request.node.get_closest_marker("only_asyncio") and reactor_pytest != "asyncio":
+        pytest.skip("This test is only run with --reactor=asyncio")
+
+
+@pytest.fixture(autouse=True)
+def only_not_asyncio(request, reactor_pytest):
+    if (
+        request.node.get_closest_marker("only_not_asyncio")
+        and reactor_pytest == "asyncio"
+    ):
+        pytest.skip("This test is only run without --reactor=asyncio")
 
 
 def pytest_configure(config):
