@@ -1,32 +1,32 @@
 import sys
-import os
 from importlib import import_module
+from os import PathLike
+from pathlib import Path
+from types import ModuleType
+from typing import Union
 
-from scrapy.utils.spider import iter_spider_classes
-from scrapy.exceptions import UsageError
 from scrapy.commands import BaseRunSpiderCommand
+from scrapy.exceptions import UsageError
+from scrapy.utils.spider import iter_spider_classes
 
 
-def _import_file(filepath):
-    abspath = os.path.abspath(filepath)
-    dirname, file = os.path.split(abspath)
-    fname, fext = os.path.splitext(file)
-    if fext not in ('.py', '.pyw'):
+def _import_file(filepath: Union[str, PathLike]) -> ModuleType:
+    abspath = Path(filepath).resolve()
+    if abspath.suffix not in (".py", ".pyw"):
         raise ValueError(f"Not a Python source file: {abspath}")
-    if dirname:
-        sys.path = [dirname] + sys.path
+    dirname = str(abspath.parent)
+    sys.path = [dirname] + sys.path
     try:
-        module = import_module(fname)
+        module = import_module(abspath.stem)
     finally:
-        if dirname:
-            sys.path.pop(0)
+        sys.path.pop(0)
     return module
 
 
 class Command(BaseRunSpiderCommand):
 
     requires_project = False
-    default_settings = {'SPIDER_LOADER_WARN_ONLY': True}
+    default_settings = {"SPIDER_LOADER_WARN_ONLY": True}
 
     def syntax(self):
         return "[options] <spider_file>"
@@ -40,13 +40,13 @@ class Command(BaseRunSpiderCommand):
     def run(self, args, opts):
         if len(args) != 1:
             raise UsageError()
-        filename = args[0]
-        if not os.path.exists(filename):
+        filename = Path(args[0])
+        if not filename.exists():
             raise UsageError(f"File not found: {filename}\n")
         try:
             module = _import_file(filename)
         except (ImportError, ValueError) as e:
-            raise UsageError(f"Unable to load {filename!r}: {e}\n")
+            raise UsageError(f"Unable to load {str(filename)!r}: {e}\n")
         spclasses = list(iter_spider_classes(module))
         if not spclasses:
             raise UsageError(f"No spider found in file: {filename}\n")
