@@ -5,8 +5,7 @@ from scrapy.utils.versions import scrapy_components_versions
 
 class Command(ScrapyCommand):
 
-    default_settings = {'LOG_ENABLED': False,
-                        'SPIDER_LOADER_WARN_ONLY': True}
+    default_settings = {"LOG_ENABLED": False, "SPIDER_LOADER_WARN_ONLY": True}
 
     def syntax(self):
         return "[-v]"
@@ -16,8 +15,13 @@ class Command(ScrapyCommand):
 
     def add_options(self, parser):
         ScrapyCommand.add_options(self, parser)
-        parser.add_argument("--verbose", "-v", dest="verbose", action="store_true",
-                            help="also display twisted/python/platform info (useful for bug reports)")
+        parser.add_argument(
+            "--verbose",
+            "-v",
+            dest="verbose",
+            action="store_true",
+            help="also display twisted/python/platform info (useful for bug reports)",
+        )
 
     def run(self, args, opts):
         if opts.verbose:
