@@ -8,8 +8,10 @@ class HTTP10DownloadHandler:
     lazy = False
 
     def __init__(self, settings, crawler=None):
-        self.HTTPClientFactory = load_object(settings['DOWNLOADER_HTTPCLIENTFACTORY'])
-        self.ClientContextFactory = load_object(settings['DOWNLOADER_CLIENTCONTEXTFACTORY'])
+        self.HTTPClientFactory = load_object(settings["DOWNLOADER_HTTPCLIENTFACTORY"])
+        self.ClientContextFactory = load_object(
+            settings["DOWNLOADER_CLIENTCONTEXTFACTORY"]
+        )
         self._settings = settings
         self._crawler = crawler
 
@@ -25,13 +27,13 @@ class HTTP10DownloadHandler:
 
     def _connect(self, factory):
         from twisted.internet import reactor
+
         host, port = to_unicode(factory.host), factory.port
-        if factory.scheme == b'https':
+        if factory.scheme == b"https":
             client_context_factory = create_instance(
                 objcls=self.ClientContextFactory,
                 settings=self._settings,
                 crawler=self._crawler,
             )
             return reactor.connectSSL(host, port, factory, client_context_factory)
-        else:
-            return reactor.connectTCP(host, port, factory)
+        return reactor.connectTCP(host, port, factory)
