@@ -3,38 +3,46 @@ Downloader Middleware manager
 
 See documentation in docs/topics/downloader-middleware.rst
 """
-from twisted.internet import defer
+from typing import Callable, Union, cast
 
+from twisted.internet import defer
+from twisted.python.failure import Failure
+
+from scrapy import Spider
 from scrapy.exceptions import _InvalidOutput
 from scrapy.http import Request, Response
 from scrapy.middleware import MiddlewareManager
-from scrapy.utils.defer import mustbe_deferred, deferred_from_coro
 from scrapy.utils.conf import build_component_list
+from scrapy.utils.defer import deferred_from_coro, mustbe_deferred
 
 
 class DownloaderMiddlewareManager(MiddlewareManager):
 
-    component_name = 'downloader middleware'
+    component_name = "downloader middleware"
 
     @classmethod
     def _get_mwlist_from_settings(cls, settings):
-        return build_component_list(
-            settings.getwithbase('DOWNLOADER_MIDDLEWARES'))
+        return build_component_list(settings.getwithbase("DOWNLOADER_MIDDLEWARES"))
 
     def _add_middleware(self, mw):
-        if hasattr(mw, 'process_request'):
-            self.methods['process_request'].append(mw.process_request)
-        if hasattr(mw, 'process_response'):
-            self.methods['process_response'].appendleft(mw.process_response)
-        if hasattr(mw, 'process_exception'):
-            self.methods['process_exception'].appendleft(mw.process_exception)
+        if hasattr(mw, "process_request"):
+            self.methods["process_request"].append(mw.process_request)
+        if hasattr(mw, "process_response"):
+            self.methods["process_response"].appendleft(mw.process_response)
+        if hasattr(mw, "process_exception"):
+            self.methods["process_exception"].appendleft(mw.process_exception)
 
-    def download(self, download_func, request, spider):
+    def download(self, download_func: Callable, request: Request, spider: Spider):
         @defer.inlineCallbacks
-        def process_request(request):
-            for method in self.methods['process_request']:
-                response = yield deferred_from_coro(method(request=request, spider=spider))
-                if response is not None and not isinstance(response, (Response, Request)):
+        def process_request(request: Request):
+            for method in self.methods["process_request"]:
+                method = cast(Callable, method)
+                response = yield deferred_from_coro(
+                    method(request=request, spider=spider)
+                )
+                if response is not None and not isinstance(
+                    response, (Response, Request)
+                ):
                     raise _InvalidOutput(
                         f"Middleware {method.__qualname__} must return None, Response or "
                         f"Request, got {response.__class__.__name__}"
@@ -44,14 +52,17 @@ class DownloaderMiddlewareManager(MiddlewareManager):
             return (yield download_func(request=request, spider=spider))
 
         @defer.inlineCallbacks
-        def process_response(response):
+        def process_response(response: Union[Response, Request]):
             if response is None:
                 raise TypeError("Received None in process_response")
             elif isinstance(response, Request):
                 return response
 
-            for method in self.methods['process_response']:
-                response = yield deferred_from_coro(method(request=request, response=response, spider=spider))
+            for method in self.methods["process_response"]:
+                method = cast(Callable, method)
+                response = yield deferred_from_coro(
+                    method(request=request, response=response, spider=spider)
+                )
                 if not isinstance(response, (Response, Request)):
                     raise _InvalidOutput(
                         f"Middleware {method.__qualname__} must return Response or Request, "
@@ -62,11 +73,16 @@ class DownloaderMiddlewareManager(MiddlewareManager):
             return response
 
         @defer.inlineCallbacks
-        def process_exception(failure):
+        def process_exception(failure: Failure):
             exception = failure.value
-            for method in self.methods['process_exception']:
-                response = yield deferred_from_coro(method(request=request, exception=exception, spider=spider))
-                if response is not None and not isinstance(response, (Response, Request)):
+            for method in self.methods["process_exception"]:
+                method = cast(Callable, method)
+                response = yield deferred_from_coro(
+                    method(request=request, exception=exception, spider=spider)
+                )
+                if response is not None and not isinstance(
+                    response, (Response, Request)
+                ):
                     raise _InvalidOutput(
                         f"Middleware {method.__qualname__} must return None, Response or "
                         f"Request, got {type(response)}"
