@@ -8,13 +8,12 @@ from scrapy.utils.python import without_none_values
 
 
 class DefaultHeadersMiddleware:
-
     def __init__(self, headers):
         self._headers = headers
 
     @classmethod
     def from_crawler(cls, crawler):
-        headers = without_none_values(crawler.settings['DEFAULT_REQUEST_HEADERS'])
+        headers = without_none_values(crawler.settings["DEFAULT_REQUEST_HEADERS"])
         return cls(headers.items())
 
     def process_request(self, request, spider):
