@@ -9,7 +9,7 @@ RETRY_HTTP_CODES - which HTTP response codes to retry
 Failed pages are collected on the scraping process and rescheduled at the end,
 once the spider has finished crawling all regular (non failed) pages.
 """
-from logging import getLogger, Logger
+from logging import Logger, getLogger
 from typing import Optional, Union
 
 from twisted.internet import defer
@@ -31,7 +31,6 @@ from scrapy.spiders import Spider
 from scrapy.utils.python import global_object_name
 from scrapy.utils.response import response_status_message
 
-
 retry_logger = getLogger(__name__)
 
 
@@ -39,11 +38,11 @@ def get_retry_request(
     request: Request,
     *,
     spider: Spider,
-    reason: Union[str, Exception] = 'unspecified',
+    reason: Union[str, Exception] = "unspecified",
     max_retry_times: Optional[int] = None,
     priority_adjust: Optional[int] = None,
     logger: Logger = retry_logger,
-    stats_base_key: str = 'retry',
+    stats_base_key: str = "retry",
 ):
     """
     Returns a new :class:`~scrapy.Request` object to retry the specified
@@ -87,22 +86,22 @@ def get_retry_request(
     """
     settings = spider.crawler.settings
     stats = spider.crawler.stats
-    retry_times = request.meta.get('retry_times', 0) + 1
+    retry_times = request.meta.get("retry_times", 0) + 1
     if max_retry_times is None:
-        max_retry_times = request.meta.get('max_retry_times')
+        max_retry_times = request.meta.get("max_retry_times")
         if max_retry_times is None:
-            max_retry_times = settings.getint('RETRY_TIMES')
+            max_retry_times = settings.getint("RETRY_TIMES")
     if retry_times <= max_retry_times:
         logger.debug(
             "Retrying %(request)s (failed %(retry_times)d times): %(reason)s",
-            {'request': request, 'retry_times': retry_times, 'reason': reason},
-            extra={'spider': spider}
+            {"request": request, "retry_times": retry_times, "reason": reason},
+            extra={"spider": spider},
         )
         new_request: Request = request.copy()
-        new_request.meta['retry_times'] = retry_times
+        new_request.meta["retry_times"] = retry_times
         new_request.dont_filter = True
         if priority_adjust is None:
-            priority_adjust = settings.getint('RETRY_PRIORITY_ADJUST')
+            priority_adjust = settings.getint("RETRY_PRIORITY_ADJUST")
         new_request.priority = request.priority + priority_adjust
 
         if callable(reason):
@@ -110,42 +109,51 @@ def get_retry_request(
         if isinstance(reason, Exception):
             reason = global_object_name(reason.__class__)
 
-        stats.inc_value(f'{stats_base_key}/count')
-        stats.inc_value(f'{stats_base_key}/reason_count/{reason}')
+        stats.inc_value(f"{stats_base_key}/count")
+        stats.inc_value(f"{stats_base_key}/reason_count/{reason}")
         return new_request
-    else:
-        stats.inc_value(f'{stats_base_key}/max_reached')
-        logger.error(
-            "Gave up retrying %(request)s (failed %(retry_times)d times): "
-            "%(reason)s",
-            {'request': request, 'retry_times': retry_times, 'reason': reason},
-            extra={'spider': spider},
-        )
-        return None
+    stats.inc_value(f"{stats_base_key}/max_reached")
+    logger.error(
+        "Gave up retrying %(request)s (failed %(retry_times)d times): " "%(reason)s",
+        {"request": request, "retry_times": retry_times, "reason": reason},
+        extra={"spider": spider},
+    )
+    return None
 
 
 class RetryMiddleware:
 
     # IOError is raised by the HttpCompression middleware when trying to
     # decompress an empty response
-    EXCEPTIONS_TO_RETRY = (defer.TimeoutError, TimeoutError, DNSLookupError,
-                           ConnectionRefusedError, ConnectionDone, ConnectError,
-                           ConnectionLost, TCPTimedOutError, ResponseFailed,
-                           IOError, TunnelError)
+    EXCEPTIONS_TO_RETRY = (
+        defer.TimeoutError,
+        TimeoutError,
+        DNSLookupError,
+        ConnectionRefusedError,
+        ConnectionDone,
+        ConnectError,
+        ConnectionLost,
+        TCPTimedOutError,
+        ResponseFailed,
+        IOError,
+        TunnelError,
+    )
 
     def __init__(self, settings):
-        if not settings.getbool('RETRY_ENABLED'):
+        if not settings.getbool("RETRY_ENABLED"):
             raise NotConfigured
-        self.max_retry_times = settings.getint('RETRY_TIMES')
-        self.retry_http_codes = set(int(x) for x in settings.getlist('RETRY_HTTP_CODES'))
-        self.priority_adjust = settings.getint('RETRY_PRIORITY_ADJUST')
+        self.max_retry_times = settings.getint("RETRY_TIMES")
+        self.retry_http_codes = set(
+            int(x) for x in settings.getlist("RETRY_HTTP_CODES")
+        )
+        self.priority_adjust = settings.getint("RETRY_PRIORITY_ADJUST")
 
     @classmethod
     def from_crawler(cls, crawler):
         return cls(crawler.settings)
 
     def process_response(self, request, response, spider):
-        if request.meta.get('dont_retry', False):
+        if request.meta.get("dont_retry", False):
             return response
         if response.status in self.retry_http_codes:
             reason = response_status_message(response.status)
@@ -153,15 +161,14 @@ class RetryMiddleware:
         return response
 
     def process_exception(self, request, exception, spider):
-        if (
-            isinstance(exception, self.EXCEPTIONS_TO_RETRY)
-            and not request.meta.get('dont_retry', False)
+        if isinstance(exception, self.EXCEPTIONS_TO_RETRY) and not request.meta.get(
+            "dont_retry", False
         ):
             return self._retry(request, exception, spider)
 
     def _retry(self, request, reason, spider):
-        max_retry_times = request.meta.get('max_retry_times', self.max_retry_times)
-        priority_adjust = request.meta.get('priority_adjust', self.priority_adjust)
+        max_retry_times = request.meta.get("max_retry_times", self.max_retry_times)
+        priority_adjust = request.meta.get("priority_adjust", self.priority_adjust)
         return get_retry_request(
             request,
             reason=reason,
