@@ -15,7 +15,7 @@ def wrap_loader_context(function, context):
         "scrapy.loader.common.wrap_loader_context has moved to a new library."
         "Please update your reference to itemloaders.common.wrap_loader_context",
         ScrapyDeprecationWarning,
-        stacklevel=2
+        stacklevel=2,
     )
 
     return common.wrap_loader_context(function, context)
