@@ -3,24 +3,23 @@ XPath selectors based on lxml
 """
 
 from parsel import Selector as _ParselSelector
-from scrapy.utils.trackref import object_ref
-from scrapy.utils.python import to_bytes
+
 from scrapy.http import HtmlResponse, XmlResponse
+from scrapy.utils.python import to_bytes
+from scrapy.utils.trackref import object_ref
 
-
-__all__ = ['Selector', 'SelectorList']
+__all__ = ["Selector", "SelectorList"]
 
 
 def _st(response, st):
     if st is None:
-        return 'xml' if isinstance(response, XmlResponse) else 'html'
+        return "xml" if isinstance(response, XmlResponse) else "html"
     return st
 
 
 def _response_from_text(text, st):
-    rt = XmlResponse if st == 'xml' else HtmlResponse
-    return rt(url='about:blank', encoding='utf-8',
-              body=to_bytes(text, 'utf-8'))
+    rt = XmlResponse if st == "xml" else HtmlResponse
+    return rt(url="about:blank", encoding="utf-8", body=to_bytes(text, "utf-8"))
 
 
 class SelectorList(_ParselSelector.selectorlist_cls, object_ref):
@@ -61,22 +60,24 @@ class Selector(_ParselSelector, object_ref):
     detection will occur.
     """
 
-    __slots__ = ['response']
+    __slots__ = ["response"]
     selectorlist_cls = SelectorList
 
     def __init__(self, response=None, text=None, type=None, root=None, **kwargs):
         if response is not None and text is not None:
-            raise ValueError(f'{self.__class__.__name__}.__init__() received '
-                             'both response and text')
+            raise ValueError(
+                f"{self.__class__.__name__}.__init__() received "
+                "both response and text"
+            )
 
-        st = _st(response, type or self._default_type)
+        st = _st(response, type)
 
         if text is not None:
             response = _response_from_text(text, st)
 
         if response is not None:
             text = response.text
-            kwargs.setdefault('base_url', response.url)
+            kwargs.setdefault("base_url", response.url)
 
         self.response = response
         super().__init__(text=text, type=st, root=root, **kwargs)
