@@ -5,9 +5,10 @@ pprint and pformat wrappers with colorization support
 import ctypes
 import platform
 import sys
-from distutils.version import LooseVersion as parse_version
 from pprint import pformat as pformat_
 
+from packaging.version import Version as parse_version
+
 
 def _enable_windows_terminal_processing():
     # https://stackoverflow.com/a/36760881
@@ -37,11 +38,12 @@ def _colorize(text, colorize=True):
     else:
         from pygments.formatters import TerminalFormatter
         from pygments.lexers import PythonLexer
+
         return highlight(text, PythonLexer(), TerminalFormatter())
 
 
 def pformat(obj, *args, **kwargs):
-    return _colorize(pformat_(obj), kwargs.pop('colorize', True))
+    return _colorize(pformat_(obj), kwargs.pop("colorize", True))
 
 
 def pprint(obj, *args, **kwargs):
