@@ -1,25 +1,24 @@
 """Helper functions which don't fit anywhere else"""
 import ast
+import hashlib
 import inspect
 import os
 import re
-import hashlib
 import warnings
 from collections import deque
 from contextlib import contextmanager
+from functools import partial
 from importlib import import_module
 from pkgutil import iter_modules
-from textwrap import dedent
 
 from w3lib.html import replace_entities
 
+from scrapy.item import Item
 from scrapy.utils.datatypes import LocalWeakReferencedCache
-from scrapy.utils.python import flatten, to_unicode
-from scrapy.item import _BaseItem
 from scrapy.utils.deprecate import ScrapyDeprecationWarning
+from scrapy.utils.python import flatten, to_unicode
 
-
-_ITERABLE_SINGLE_VALUES = dict, _BaseItem, str, bytes
+_ITERABLE_SINGLE_VALUES = dict, Item, str, bytes
 
 
 def arg_to_iter(arg):
@@ -30,10 +29,9 @@ def arg_to_iter(arg):
     """
     if arg is None:
         return []
-    elif not isinstance(arg, _ITERABLE_SINGLE_VALUES) and hasattr(arg, '__iter__'):
+    if not isinstance(arg, _ITERABLE_SINGLE_VALUES) and hasattr(arg, "__iter__"):
         return arg
-    else:
-        return [arg]
+    return [arg]
 
 
 def load_object(path):
@@ -49,16 +47,16 @@ def load_object(path):
     if not isinstance(path, str):
         if callable(path):
             return path
-        else:
-            raise TypeError("Unexpected argument type, expected string "
-                            "or object, got: %s" % type(path))
+        raise TypeError(
+            "Unexpected argument type, expected string " f"or object, got: {type(path)}"
+        )
 
     try:
-        dot = path.rindex('.')
+        dot = path.rindex(".")
     except ValueError:
         raise ValueError(f"Error loading object '{path}': not a full path")
 
-    module, name = path[:dot], path[dot + 1:]
+    module, name = path[:dot], path[dot + 1 :]
     mod = import_module(module)
 
     try:
@@ -80,9 +78,9 @@ def walk_modules(path):
     mods = []
     mod = import_module(path)
     mods.append(mod)
-    if hasattr(mod, '__path__'):
+    if hasattr(mod, "__path__"):
         for _, subpath, ispkg in iter_modules(mod.__path__):
-            fullpath = path + '.' + subpath
+            fullpath = path + "." + subpath
             if ispkg:
                 mods += walk_modules(fullpath)
             else:
@@ -91,7 +89,7 @@ def walk_modules(path):
     return mods
 
 
-def extract_regex(regex, text, encoding='utf-8'):
+def extract_regex(regex, text, encoding="utf-8"):
     """Extract a list of unicode strings from the given text/encoding using the following policies:
 
     * if the regex contains a named group called "extract" that will be returned
@@ -101,23 +99,23 @@ def extract_regex(regex, text, encoding='utf-8'):
     warnings.warn(
         "scrapy.utils.misc.extract_regex has moved to parsel.utils.extract_regex.",
         ScrapyDeprecationWarning,
-        stacklevel=2
+        stacklevel=2,
     )
 
     if isinstance(regex, str):
         regex = re.compile(regex, re.UNICODE)
 
     try:
-        strings = [regex.search(text).group('extract')]   # named group
+        strings = [regex.search(text).group("extract")]  # named group
     except Exception:
-        strings = regex.findall(text)    # full regex or numbered groups
+        strings = regex.findall(text)  # full regex or numbered groups
     strings = flatten(strings)
 
     if isinstance(text, str):
-        return [replace_entities(s, keep=['lt', 'amp']) for s in strings]
-    else:
-        return [replace_entities(to_unicode(s, encoding), keep=['lt', 'amp'])
-                for s in strings]
+        return [replace_entities(s, keep=["lt", "amp"]) for s in strings]
+    return [
+        replace_entities(to_unicode(s, encoding), keep=["lt", "amp"]) for s in strings
+    ]
 
 
 def md5sum(file):
@@ -139,7 +137,7 @@ def md5sum(file):
 
 def rel_has_nofollow(rel):
     """Return True if link rel attribute has nofollow type"""
-    return rel is not None and 'nofollow' in rel.split()
+    return rel is not None and "nofollow" in rel.replace(",", " ").split()
 
 
 def create_instance(objcls, settings, crawler, *args, **kwargs):
@@ -163,15 +161,15 @@ def create_instance(objcls, settings, crawler, *args, **kwargs):
         if crawler is None:
             raise ValueError("Specify at least one of settings and crawler.")
         settings = crawler.settings
-    if crawler and hasattr(objcls, 'from_crawler'):
+    if crawler and hasattr(objcls, "from_crawler"):
         instance = objcls.from_crawler(crawler, *args, **kwargs)
-        method_name = 'from_crawler'
-    elif hasattr(objcls, 'from_settings'):
+        method_name = "from_crawler"
+    elif hasattr(objcls, "from_settings"):
         instance = objcls.from_settings(settings, *args, **kwargs)
-        method_name = 'from_settings'
+        method_name = "from_settings"
     else:
         instance = objcls(*args, **kwargs)
-        method_name = '__new__'
+        method_name = "__new__"
     if instance is None:
         raise TypeError(f"{objcls.__qualname__}.{method_name} returned None")
     return instance
@@ -224,10 +222,24 @@ def is_generator_with_return_value(callable):
 
     def returns_none(return_node):
         value = return_node.value
-        return value is None or isinstance(value, ast.NameConstant) and value.value is None
+        return (
+            value is None or isinstance(value, ast.NameConstant) and value.value is None
+        )
 
     if inspect.isgeneratorfunction(callable):
-        tree = ast.parse(dedent(inspect.getsource(callable)))
+        func = callable
+        while isinstance(func, partial):
+            func = func.func
+
+        src = inspect.getsource(func)
+        pattern = re.compile(r"(^[\t ]+)")
+        code = pattern.sub("", src)
+
+        match = pattern.match(src)  # finds indentation
+        if match:
+            code = re.sub(f"\n{match.group(0)}", "\n", code)  # remove indentation
+
+        tree = ast.parse(code)
         for node in walk_callable(tree):
             if isinstance(node, ast.Return) and not returns_none(node):
                 _generator_callbacks_cache[callable] = True
@@ -242,12 +254,23 @@ def warn_on_generator_with_return_value(spider, callable):
     Logs a warning if a callable is a generator function and includes
     a 'return' statement with a value different than None
     """
-    if is_generator_with_return_value(callable):
+    try:
+        if is_generator_with_return_value(callable):
+            warnings.warn(
+                f'The "{spider.__class__.__name__}.{callable.__name__}" method is '
+                'a generator and includes a "return" statement with a value '
+                "different than None. This could lead to unexpected behaviour. Please see "
+                "https://docs.python.org/3/reference/simple_stmts.html#the-return-statement "
+                'for details about the semantics of the "return" statement within generators',
+                stacklevel=2,
+            )
+    except IndentationError:
+        callable_name = spider.__class__.__name__ + "." + callable.__name__
         warnings.warn(
-            f'The "{spider.__class__.__name__}.{callable.__name__}" method is '
-            'a generator and includes a "return" statement with a value '
-            'different than None. This could lead to unexpected behaviour. Please see '
-            'https://docs.python.org/3/reference/simple_stmts.html#the-return-statement '
-            'for details about the semantics of the "return" statement within generators',
+            f'Unable to determine whether or not "{callable_name}" is a generator with a return value. '
+            "This will not prevent your code from working, but it prevents Scrapy from detecting "
+            f'potential issues in your implementation of "{callable_name}". Please, report this in the '
+            "Scrapy issue tracker (https://github.com/scrapy/scrapy/issues), "
+            f'including the code of "{callable_name}"',
             stacklevel=2,
         )
