@@ -5,7 +5,6 @@ library.
 Some of the functions that used to be imported from this module have been moved
 to the w3lib.url module. Always import those from there instead.
 """
-import posixpath
 import re
 from urllib.parse import ParseResult, urldefrag, urlparse, urlunparse
 
@@ -13,6 +12,7 @@ from urllib.parse import ParseResult, urldefrag, urlparse, urlunparse
 # move doesn't break old code
 from w3lib.url import *
 from w3lib.url import _safe_chars, _unquotepath  # noqa: F401
+
 from scrapy.utils.python import to_unicode
 
 
@@ -22,16 +22,20 @@ def url_is_from_any_domain(url, domains):
     if not host:
         return False
     domains = [d.lower() for d in domains]
-    return any((host == d) or (host.endswith(f'.{d}')) for d in domains)
+    return any((host == d) or (host.endswith(f".{d}")) for d in domains)
 
 
 def url_is_from_spider(url, spider):
     """Return True if the url belongs to the given spider"""
-    return url_is_from_any_domain(url, [spider.name] + list(getattr(spider, 'allowed_domains', [])))
+    return url_is_from_any_domain(
+        url, [spider.name] + list(getattr(spider, "allowed_domains", []))
+    )
 
 
 def url_has_any_extension(url, extensions):
-    return posixpath.splitext(parse_url(url).path)[1].lower() in extensions
+    """Return True if the url ends with one of the extensions provided"""
+    lowercase_path = parse_url(url).path.lower()
+    return any(lowercase_path.endswith(ext) for ext in extensions)
 
 
 def parse_url(url, encoding=None):
@@ -45,7 +49,7 @@ def parse_url(url, encoding=None):
 
 def escape_ajax(url):
     """
-    Return the crawleable url according to:
+    Return the crawlable url according to:
     https://developers.google.com/webmasters/ajax-crawling/docs/getting-started
 
     >>> escape_ajax("www.example.com/ajax.html#!key=value")
@@ -67,9 +71,9 @@ def escape_ajax(url):
     'www.example.com/ajax.html'
     """
     defrag, frag = urldefrag(url)
-    if not frag.startswith('!'):
+    if not frag.startswith("!"):
         return url
-    return add_or_replace_parameter(defrag, '_escaped_fragment_', frag[1:])
+    return add_or_replace_parameter(defrag, "_escaped_fragment_", frag[1:])
 
 
 def add_http_if_no_scheme(url):
@@ -86,7 +90,7 @@ def add_http_if_no_scheme(url):
 def _is_posix_path(string):
     return bool(
         re.match(
-            r'''
+            r"""
             ^                   # start with...
             (
                 \.              # ...a single dot,
@@ -98,7 +102,7 @@ def _is_posix_path(string):
             )?      # optional match of ".", ".." or ".blabla"
             /       # at least one "/" for a file path,
             .       # and something after the "/"
-            ''',
+            """,
             string,
             flags=re.VERBOSE,
         )
@@ -108,13 +112,13 @@ def _is_posix_path(string):
 def _is_windows_path(string):
     return bool(
         re.match(
-            r'''
+            r"""
             ^
             (
                 [a-z]:\\
                 | \\\\
             )
-            ''',
+            """,
             string,
             flags=re.IGNORECASE | re.VERBOSE,
         )
@@ -133,7 +137,13 @@ def guess_scheme(url):
     return add_http_if_no_scheme(url)
 
 
-def strip_url(url, strip_credentials=True, strip_default_port=True, origin_only=False, strip_fragment=True):
+def strip_url(
+    url,
+    strip_credentials=True,
+    strip_default_port=True,
+    origin_only=False,
+    strip_fragment=True,
+):
 
     """Strip URL string from some of its components:
 
@@ -147,18 +157,24 @@ def strip_url(url, strip_credentials=True, strip_default_port=True, origin_only=
 
     parsed_url = urlparse(url)
     netloc = parsed_url.netloc
-    if (strip_credentials or origin_only) and (parsed_url.username or parsed_url.password):
-        netloc = netloc.split('@')[-1]
+    if (strip_credentials or origin_only) and (
+        parsed_url.username or parsed_url.password
+    ):
+        netloc = netloc.split("@")[-1]
     if strip_default_port and parsed_url.port:
-        if (parsed_url.scheme, parsed_url.port) in (('http', 80),
-                                                    ('https', 443),
-                                                    ('ftp', 21)):
-            netloc = netloc.replace(f':{parsed_url.port}', '')
-    return urlunparse((
-        parsed_url.scheme,
-        netloc,
-        '/' if origin_only else parsed_url.path,
-        '' if origin_only else parsed_url.params,
-        '' if origin_only else parsed_url.query,
-        '' if strip_fragment else parsed_url.fragment
-    ))
+        if (parsed_url.scheme, parsed_url.port) in (
+            ("http", 80),
+            ("https", 443),
+            ("ftp", 21),
+        ):
+            netloc = netloc.replace(f":{parsed_url.port}", "")
+    return urlunparse(
+        (
+            parsed_url.scheme,
+            netloc,
+            "/" if origin_only else parsed_url.path,
+            "" if origin_only else parsed_url.params,
+            "" if origin_only else parsed_url.query,
+            "" if strip_fragment else parsed_url.fragment,
+        )
+    )
