@@ -5,7 +5,6 @@ from typing import Optional
 from scrapy import Spider
 from scrapy.crawler import CrawlerProcess
 from scrapy.utils.defer import deferred_from_coro
-from twisted.internet.defer import Deferred
 
 
 class UppercasePipeline:
@@ -38,9 +37,11 @@ if __name__ == "__main__":
     except IndexError:
         ASYNCIO_EVENT_LOOP = None
 
-    process = CrawlerProcess(settings={
-        "TWISTED_REACTOR": "twisted.internet.asyncioreactor.AsyncioSelectorReactor",
-        "ASYNCIO_EVENT_LOOP": ASYNCIO_EVENT_LOOP,
-    })
+    process = CrawlerProcess(
+        settings={
+            "TWISTED_REACTOR": "twisted.internet.asyncioreactor.AsyncioSelectorReactor",
+            "ASYNCIO_EVENT_LOOP": ASYNCIO_EVENT_LOOP,
+        }
+    )
     process.crawl(UrlSpider)
     process.start()
