@@ -1,10 +1,11 @@
+from twisted.internet import reactor  # noqa: F401
+
 import scrapy
 from scrapy.crawler import CrawlerProcess
-from twisted.internet import reactor
 
 
 class NoRequestsSpider(scrapy.Spider):
-    name = 'no_request'
+    name = "no_request"
 
     def start_requests(self):
         return []
@@ -14,4 +15,3 @@ process = CrawlerProcess(settings={})
 
 process.crawl(NoRequestsSpider)
 process.start()
-
