@@ -1,20 +1,21 @@
+from twisted.internet import reactor  # noqa: F401
+
 import scrapy
 from scrapy.crawler import CrawlerProcess
-from twisted.internet import reactor
 
 
 class NoRequestsSpider(scrapy.Spider):
-    name = 'no_request'
+    name = "no_request"
 
     def start_requests(self):
         return []
 
 
-process = CrawlerProcess(settings={
-    "TWISTED_REACTOR": "twisted.internet.selectreactor.SelectReactor",
-})
+process = CrawlerProcess(
+    settings={
+        "TWISTED_REACTOR": "twisted.internet.selectreactor.SelectReactor",
+    }
+)
 
 process.crawl(NoRequestsSpider)
 process.start()
-
-
