@@ -4,6 +4,7 @@ import collections
 
 class DummyDB(dict):
     """Provide dummy DBM-like interface."""
+
     def close(self):
         pass
 
@@ -14,7 +15,7 @@ error = KeyError
 _DATABASES = collections.defaultdict(DummyDB)
 
 
-def open(file, flag='r', mode=0o666):
+def open(file, flag="r", mode=0o666):
     """Open or create a dummy database compatible.
 
     Arguments ``flag`` and ``mode`` are ignored.
