@@ -26,26 +26,28 @@ class MockServerSpider(Spider):
 
 class MetaSpider(MockServerSpider):
 
-    name = 'meta'
+    name = "meta"
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.meta = {}
 
     def closed(self, reason):
-        self.meta['close_reason'] = reason
+        self.meta["close_reason"] = reason
 
 
 class FollowAllSpider(MetaSpider):
 
-    name = 'follow'
+    name = "follow"
     link_extractor = LinkExtractor()
 
-    def __init__(self, total=10, show=20, order="rand", maxlatency=0.0, *args, **kwargs):
+    def __init__(
+        self, total=10, show=20, order="rand", maxlatency=0.0, *args, **kwargs
+    ):
         super().__init__(*args, **kwargs)
         self.urls_visited = []
         self.times = []
-        qargs = {'total': total, 'show': show, 'order': order, 'maxlatency': maxlatency}
+        qargs = {"total": total, "show": show, "order": order, "maxlatency": maxlatency}
         url = self.mockserver.url(f"/follow?{urlencode(qargs, doseq=True)}")
         self.start_urls = [url]
 
@@ -58,7 +60,7 @@ class FollowAllSpider(MetaSpider):
 
 class DelaySpider(MetaSpider):
 
-    name = 'delay'
+    name = "delay"
 
     def __init__(self, n=1, b=0, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -80,7 +82,7 @@ class DelaySpider(MetaSpider):
 
 class SimpleSpider(MetaSpider):
 
-    name = 'simple'
+    name = "simple"
 
     def __init__(self, url="http://localhost:8998", *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -92,7 +94,7 @@ class SimpleSpider(MetaSpider):
 
 class AsyncDefSpider(SimpleSpider):
 
-    name = 'asyncdef'
+    name = "asyncdef"
 
     async def parse(self, response):
         await defer.succeed(42)
@@ -101,7 +103,7 @@ class AsyncDefSpider(SimpleSpider):
 
 class AsyncDefAsyncioSpider(SimpleSpider):
 
-    name = 'asyncdef_asyncio'
+    name = "asyncdef_asyncio"
 
     async def parse(self, response):
         await asyncio.sleep(0.2)
@@ -111,13 +113,13 @@ class AsyncDefAsyncioSpider(SimpleSpider):
 
 class AsyncDefAsyncioReturnSpider(SimpleSpider):
 
-    name = 'asyncdef_asyncio_return'
+    name = "asyncdef_asyncio_return"
 
     async def parse(self, response):
         await asyncio.sleep(0.2)
         status = await get_from_asyncio_queue(response.status)
         self.logger.info(f"Got response {status}")
-        return [{'id': 1}, {'id': 2}]
+        return [{"id": 1}, {"id": 2}]
 
 
 class AsyncDefAsyncioReturnSingleElementSpider(SimpleSpider):
@@ -133,113 +135,119 @@ class AsyncDefAsyncioReturnSingleElementSpider(SimpleSpider):
 
 class AsyncDefAsyncioReqsReturnSpider(SimpleSpider):
 
-    name = 'asyncdef_asyncio_reqs_return'
+    name = "asyncdef_asyncio_reqs_return"
 
     async def parse(self, response):
         await asyncio.sleep(0.2)
-        req_id = response.meta.get('req_id', 0)
+        req_id = response.meta.get("req_id", 0)
         status = await get_from_asyncio_queue(response.status)
         self.logger.info(f"Got response {status}, req_id {req_id}")
         if req_id > 0:
             return
         reqs = []
         for i in range(1, 3):
-            req = Request(self.start_urls[0], dont_filter=True, meta={'req_id': i})
+            req = Request(self.start_urls[0], dont_filter=True, meta={"req_id": i})
             reqs.append(req)
         return reqs
 
 
 class AsyncDefAsyncioGenExcSpider(SimpleSpider):
-    name = 'asyncdef_asyncio_gen_exc'
+    name = "asyncdef_asyncio_gen_exc"
 
     async def parse(self, response):
         for i in range(10):
             await asyncio.sleep(0.1)
-            yield {'foo': i}
+            yield {"foo": i}
             if i > 5:
                 raise ValueError("Stopping the processing")
 
 
 class AsyncDefDeferredDirectSpider(SimpleSpider):
-    name = 'asyncdef_deferred_direct'
+    name = "asyncdef_deferred_direct"
 
     async def parse(self, response):
         resp = await get_web_client_agent_req(self.mockserver.url("/status?n=200"))
-        yield {'code': resp.code}
+        yield {"code": resp.code}
 
 
 class AsyncDefDeferredWrappedSpider(SimpleSpider):
-    name = 'asyncdef_deferred_wrapped'
+    name = "asyncdef_deferred_wrapped"
 
     async def parse(self, response):
-        resp = await deferred_to_future(get_web_client_agent_req(self.mockserver.url("/status?n=200")))
-        yield {'code': resp.code}
+        resp = await deferred_to_future(
+            get_web_client_agent_req(self.mockserver.url("/status?n=200"))
+        )
+        yield {"code": resp.code}
 
 
 class AsyncDefDeferredMaybeWrappedSpider(SimpleSpider):
-    name = 'asyncdef_deferred_wrapped'
+    name = "asyncdef_deferred_wrapped"
 
     async def parse(self, response):
-        resp = await maybe_deferred_to_future(get_web_client_agent_req(self.mockserver.url("/status?n=200")))
-        yield {'code': resp.code}
+        resp = await maybe_deferred_to_future(
+            get_web_client_agent_req(self.mockserver.url("/status?n=200"))
+        )
+        yield {"code": resp.code}
 
 
 class AsyncDefAsyncioGenSpider(SimpleSpider):
 
-    name = 'asyncdef_asyncio_gen'
+    name = "asyncdef_asyncio_gen"
 
     async def parse(self, response):
         await asyncio.sleep(0.2)
-        yield {'foo': 42}
+        yield {"foo": 42}
         self.logger.info(f"Got response {response.status}")
 
 
 class AsyncDefAsyncioGenLoopSpider(SimpleSpider):
 
-    name = 'asyncdef_asyncio_gen_loop'
+    name = "asyncdef_asyncio_gen_loop"
 
     async def parse(self, response):
         for i in range(10):
             await asyncio.sleep(0.1)
-            yield {'foo': i}
+            yield {"foo": i}
         self.logger.info(f"Got response {response.status}")
 
 
 class AsyncDefAsyncioGenComplexSpider(SimpleSpider):
 
-    name = 'asyncdef_asyncio_gen_complex'
+    name = "asyncdef_asyncio_gen_complex"
     initial_reqs = 4
     following_reqs = 3
     depth = 2
 
     def _get_req(self, index, cb=None):
-        return Request(self.mockserver.url(f"/status?n=200&request={index}"),
-                       meta={'index': index},
-                       dont_filter=True,
-                       callback=cb)
+        return Request(
+            self.mockserver.url(f"/status?n=200&request={index}"),
+            meta={"index": index},
+            dont_filter=True,
+            callback=cb,
+        )
 
     def start_requests(self):
         for i in range(1, self.initial_reqs + 1):
             yield self._get_req(i)
 
     async def parse(self, response):
-        index = response.meta['index']
-        yield {'index': index}
-        if index < 10 ** self.depth:
+        index = response.meta["index"]
+        yield {"index": index}
+        if index < 10**self.depth:
             for new_index in range(10 * index, 10 * index + self.following_reqs):
                 yield self._get_req(new_index)
         yield self._get_req(index, cb=self.parse2)
         await asyncio.sleep(0.1)
-        yield {'index': index + 5}
+        yield {"index": index + 5}
 
     async def parse2(self, response):
         await asyncio.sleep(0.1)
-        yield {'index2': response.meta['index']}
+        yield {"index2": response.meta["index"]}
 
 
 class ItemSpider(FollowAllSpider):
 
-    name = 'item'
+    name = "item"
 
     def parse(self, response):
         for request in super().parse(response):
@@ -254,11 +262,11 @@ class DefaultError(Exception):
 
 class ErrorSpider(FollowAllSpider):
 
-    name = 'error'
+    name = "error"
     exception_cls = DefaultError
 
     def raise_exception(self):
-        raise self.exception_cls('Expected exception')
+        raise self.exception_cls("Expected exception")
 
     def parse(self, response):
         for request in super().parse(response):
@@ -280,16 +288,18 @@ class BrokenStartRequestsSpider(FollowAllSpider):
             1 / 0
 
         for s in range(100):
-            qargs = {'total': 10, 'seed': s}
+            qargs = {"total": 10, "seed": s}
             url = self.mockserver.url(f"/follow?{urlencode(qargs, doseq=True)}")
-            yield Request(url, meta={'seed': s})
+            yield Request(url, meta={"seed": s})
             if self.fail_yielding:
                 2 / 0
 
-        assert self.seedsseen, 'All start requests consumed before any download happened'
+        assert (
+            self.seedsseen
+        ), "All start requests consumed before any download happened"
 
     def parse(self, response):
-        self.seedsseen.append(response.meta.get('seed'))
+        self.seedsseen.append(response.meta.get("seed"))
         for req in super().parse(response):
             yield req
 
@@ -307,21 +317,21 @@ class SingleRequestSpider(MetaSpider):
             yield Request(self.seed, callback=self.parse, errback=self.on_error)
 
     def parse(self, response):
-        self.meta.setdefault('responses', []).append(response)
+        self.meta.setdefault("responses", []).append(response)
         if callable(self.callback_func):
             return self.callback_func(response)
-        if 'next' in response.meta:
-            return response.meta['next']
+        if "next" in response.meta:
+            return response.meta["next"]
 
     def on_error(self, failure):
-        self.meta['failure'] = failure
+        self.meta["failure"] = failure
         if callable(self.errback_func):
             return self.errback_func(failure)
 
 
 class DuplicateStartRequestsSpider(MockServerSpider):
     dont_filter = True
-    name = 'duplicatestartrequests'
+    name = "duplicatestartrequests"
     distinct_urls = 2
     dupe_factor = 3
 
@@ -343,13 +353,12 @@ class CrawlSpiderWithParseMethod(MockServerSpider, CrawlSpider):
     """
     A CrawlSpider which overrides the 'parse' method
     """
-    name = 'crawl_spider_with_parse_method'
+
+    name = "crawl_spider_with_parse_method"
     custom_settings: dict = {
-        'RETRY_HTTP_CODES': [],  # no need to retry
+        "RETRY_HTTP_CODES": [],  # no need to retry
     }
-    rules = (
-        Rule(LinkExtractor(), callback='parse', follow=True),
-    )
+    rules = (Rule(LinkExtractor(), callback="parse", follow=True),)
 
     def start_requests(self):
         test_body = b"""
@@ -365,39 +374,45 @@ class CrawlSpiderWithParseMethod(MockServerSpider, CrawlSpider):
         yield Request(url, method="POST", body=test_body)
 
     def parse(self, response, foo=None):
-        self.logger.info('[parse] status %i (foo: %s)', response.status, foo)
-        yield Request(self.mockserver.url("/status?n=202"), self.parse, cb_kwargs={"foo": "bar"})
+        self.logger.info("[parse] status %i (foo: %s)", response.status, foo)
+        yield Request(
+            self.mockserver.url("/status?n=202"), self.parse, cb_kwargs={"foo": "bar"}
+        )
 
 
 class CrawlSpiderWithAsyncCallback(CrawlSpiderWithParseMethod):
     """A CrawlSpider with an async def callback"""
-    name = 'crawl_spider_with_async_callback'
-    rules = (
-        Rule(LinkExtractor(), callback='parse_async', follow=True),
-    )
+
+    name = "crawl_spider_with_async_callback"
+    rules = (Rule(LinkExtractor(), callback="parse_async", follow=True),)
 
     async def parse_async(self, response, foo=None):
-        self.logger.info('[parse_async] status %i (foo: %s)', response.status, foo)
-        return Request(self.mockserver.url("/status?n=202"), self.parse_async, cb_kwargs={"foo": "bar"})
+        self.logger.info("[parse_async] status %i (foo: %s)", response.status, foo)
+        return Request(
+            self.mockserver.url("/status?n=202"),
+            self.parse_async,
+            cb_kwargs={"foo": "bar"},
+        )
 
 
 class CrawlSpiderWithAsyncGeneratorCallback(CrawlSpiderWithParseMethod):
     """A CrawlSpider with an async generator callback"""
-    name = 'crawl_spider_with_async_generator_callback'
-    rules = (
-        Rule(LinkExtractor(), callback='parse_async_gen', follow=True),
-    )
+
+    name = "crawl_spider_with_async_generator_callback"
+    rules = (Rule(LinkExtractor(), callback="parse_async_gen", follow=True),)
 
     async def parse_async_gen(self, response, foo=None):
-        self.logger.info('[parse_async_gen] status %i (foo: %s)', response.status, foo)
-        yield Request(self.mockserver.url("/status?n=202"), self.parse_async_gen, cb_kwargs={"foo": "bar"})
+        self.logger.info("[parse_async_gen] status %i (foo: %s)", response.status, foo)
+        yield Request(
+            self.mockserver.url("/status?n=202"),
+            self.parse_async_gen,
+            cb_kwargs={"foo": "bar"},
+        )
 
 
 class CrawlSpiderWithErrback(CrawlSpiderWithParseMethod):
-    name = 'crawl_spider_with_errback'
-    rules = (
-        Rule(LinkExtractor(), callback='parse', errback='errback', follow=True),
-    )
+    name = "crawl_spider_with_errback"
+    rules = (Rule(LinkExtractor(), callback="parse", errback="errback", follow=True),)
 
     def start_requests(self):
         test_body = b"""
@@ -416,7 +431,23 @@ class CrawlSpiderWithErrback(CrawlSpiderWithParseMethod):
         yield Request(url, method="POST", body=test_body)
 
     def errback(self, failure):
-        self.logger.info('[errback] status %i', failure.value.response.status)
+        self.logger.info("[errback] status %i", failure.value.response.status)
+
+
+class CrawlSpiderWithProcessRequestCallbackKeywordArguments(CrawlSpiderWithParseMethod):
+    name = "crawl_spider_with_process_request_cb_kwargs"
+    rules = (
+        Rule(
+            LinkExtractor(),
+            callback="parse",
+            follow=True,
+            process_request="process_request",
+        ),
+    )
+
+    def process_request(self, request, response):
+        request.cb_kwargs["foo"] = "process_request"
+        return request
 
 
 class BytesReceivedCallbackSpider(MetaSpider):
@@ -446,14 +477,12 @@ class BytesReceivedCallbackSpider(MetaSpider):
 
 
 class BytesReceivedErrbackSpider(BytesReceivedCallbackSpider):
-
     def bytes_received(self, data, request, spider):
         self.meta["bytes_received"] = data
         raise StopDownload(fail=True)
 
 
 class HeadersReceivedCallbackSpider(MetaSpider):
-
     @classmethod
     def from_crawler(cls, crawler, *args, **kwargs):
         spider = super().from_crawler(crawler, *args, **kwargs)
@@ -475,7 +504,6 @@ class HeadersReceivedCallbackSpider(MetaSpider):
 
 
 class HeadersReceivedErrbackSpider(HeadersReceivedCallbackSpider):
-
     def headers_received(self, headers, body_length, request, spider):
         self.meta["headers_received"] = headers
         raise StopDownload(fail=True)
