@@ -1,39 +1,39 @@
-from os.path import join, abspath
-
 from tests.test_commands import CommandTest
 
 
 class CheckCommandTest(CommandTest):
 
-    command = 'check'
+    command = "check"
 
     def setUp(self):
-        super(CheckCommandTest, self).setUp()
-        self.spider_name = 'check_spider'
-        self.spider = abspath(join(self.proj_mod_path, 'spiders', 'checkspider.py'))
+        super().setUp()
+        self.spider_name = "check_spider"
+        self.spider = (self.proj_mod_path / "spiders" / "checkspider.py").resolve()
 
     def _write_contract(self, contracts, parse_def):
-        with open(self.spider, 'w') as file:
-            file.write(f"""
+        self.spider.write_text(
+            f"""
 import scrapy
 
 class CheckSpider(scrapy.Spider):
     name = '{self.spider_name}'
-    start_urls = ['http://example.com']
+    start_urls = ['http://toscrape.com']
 
     def parse(self, response, **cb_kwargs):
         \"\"\"
-        @url http://example.com
+        @url http://toscrape.com
         {contracts}
         \"\"\"
         {parse_def}
-            """)
+        """,
+            encoding="utf-8",
+        )
 
-    def _test_contract(self, contracts='', parse_def='pass'):
+    def _test_contract(self, contracts="", parse_def="pass"):
         self._write_contract(contracts, parse_def)
-        p, out, err = self.proc('check')
-        self.assertNotIn('F', out)
-        self.assertIn('OK', err)
+        p, out, err = self.proc("check")
+        self.assertNotIn("F", out)
+        self.assertIn("OK", err)
         self.assertEqual(p.returncode, 0)
 
     def test_check_returns_requests_contract(self):
