@@ -5,18 +5,18 @@ from twisted.python import failure
 from twisted.trial import unittest
 
 from scrapy import FormRequest
-from scrapy.crawler import CrawlerRunner
-from scrapy.spidermiddlewares.httperror import HttpError
-from scrapy.spiders import Spider
-from scrapy.http import Request
-from scrapy.item import Item, Field
-from scrapy.contracts import ContractsManager, Contract
+from scrapy.contracts import Contract, ContractsManager
 from scrapy.contracts.default import (
-    UrlContract,
     CallbackKeywordArgumentsContract,
     ReturnsContract,
     ScrapesContract,
+    UrlContract,
 )
+from scrapy.http import Request
+from scrapy.item import Field, Item
+from scrapy.spidermiddlewares.httperror import HttpError
+from scrapy.spiders import Spider
+from scrapy.utils.test import get_crawler
 from tests.mockserver import MockServer
 
 
@@ -26,52 +26,52 @@ class TestItem(Item):
 
 
 class ResponseMock:
-    url = 'http://scrapy.org'
+    url = "http://scrapy.org"
 
 
 class CustomSuccessContract(Contract):
-    name = 'custom_success_contract'
+    name = "custom_success_contract"
 
     def adjust_request_args(self, args):
-        args['url'] = 'http://scrapy.org'
+        args["url"] = "http://scrapy.org"
         return args
 
 
 class CustomFailContract(Contract):
-    name = 'custom_fail_contract'
+    name = "custom_fail_contract"
 
     def adjust_request_args(self, args):
-        raise TypeError('Error in adjust_request_args')
+        raise TypeError("Error in adjust_request_args")
 
 
 class CustomFormContract(Contract):
-    name = 'custom_form'
+    name = "custom_form"
     request_cls = FormRequest
 
     def adjust_request_args(self, args):
-        args['formdata'] = {'name': 'scrapy'}
+        args["formdata"] = {"name": "scrapy"}
         return args
 
 
 class TestSpider(Spider):
-    name = 'demo_spider'
+    name = "demo_spider"
 
     def returns_request(self, response):
-        """ method which returns request
+        """method which returns request
         @url http://scrapy.org
         @returns requests 1
         """
-        return Request('http://scrapy.org', callback=self.returns_item)
+        return Request("http://scrapy.org", callback=self.returns_item)
 
     def returns_item(self, response):
-        """ method which returns item
+        """method which returns item
         @url http://scrapy.org
         @returns items 1 1
         """
         return TestItem(url=response.url)
 
     def returns_request_cb_kwargs(self, response, url):
-        """ method which returns request
+        """method which returns request
         @url https://example.org
         @cb_kwargs {"url": "http://scrapy.org"}
         @returns requests 1
@@ -79,7 +79,7 @@ class TestSpider(Spider):
         return Request(url, callback=self.returns_item_cb_kwargs)
 
     def returns_item_cb_kwargs(self, response, name):
-        """ method which returns item
+        """method which returns item
         @url http://scrapy.org
         @cb_kwargs {"name": "Scrapy"}
         @returns items 1 1
@@ -87,7 +87,7 @@ class TestSpider(Spider):
         return TestItem(name=name, url=response.url)
 
     def returns_item_cb_kwargs_error_unexpected_keyword(self, response):
-        """ method which returns item
+        """method which returns item
         @url http://scrapy.org
         @cb_kwargs {"arg": "value"}
         @returns items 1 1
@@ -95,51 +95,51 @@ class TestSpider(Spider):
         return TestItem(url=response.url)
 
     def returns_item_cb_kwargs_error_missing_argument(self, response, arg):
-        """ method which returns item
+        """method which returns item
         @url http://scrapy.org
         @returns items 1 1
         """
         return TestItem(url=response.url)
 
     def returns_dict_item(self, response):
-        """ method which returns item
+        """method which returns item
         @url http://scrapy.org
         @returns items 1 1
         """
         return {"url": response.url}
 
     def returns_fail(self, response):
-        """ method which returns item
+        """method which returns item
         @url http://scrapy.org
         @returns items 0 0
         """
         return TestItem(url=response.url)
 
     def returns_dict_fail(self, response):
-        """ method which returns item
+        """method which returns item
         @url http://scrapy.org
         @returns items 0 0
         """
-        return {'url': response.url}
+        return {"url": response.url}
 
     def scrapes_item_ok(self, response):
-        """ returns item with name and url
+        """returns item with name and url
         @url http://scrapy.org
         @returns items 1 1
         @scrapes name url
         """
-        return TestItem(name='test', url=response.url)
+        return TestItem(name="test", url=response.url)
 
     def scrapes_dict_item_ok(self, response):
-        """ returns item with name and url
+        """returns item with name and url
         @url http://scrapy.org
         @returns items 1 1
         @scrapes name url
         """
-        return {'name': 'test', 'url': response.url}
+        return {"name": "test", "url": response.url}
 
     def scrapes_item_fail(self, response):
-        """ returns item with no name
+        """returns item with no name
         @url http://scrapy.org
         @returns items 1 1
         @scrapes name url
@@ -147,15 +147,15 @@ class TestSpider(Spider):
         return TestItem(url=response.url)
 
     def scrapes_dict_item_fail(self, response):
-        """ returns item with no name
+        """returns item with no name
         @url http://scrapy.org
         @returns items 1 1
         @scrapes name url
         """
-        return {'url': response.url}
+        return {"url": response.url}
 
     def scrapes_multiple_missing_fields(self, response):
-        """ returns item with no name
+        """returns item with no name
         @url http://scrapy.org
         @returns items 1 1
         @scrapes name url
@@ -163,7 +163,7 @@ class TestSpider(Spider):
         return {}
 
     def parse_no_url(self, response):
-        """ method with no url
+        """method with no url
         @returns items 1 1
         """
         pass
@@ -177,7 +177,7 @@ class TestSpider(Spider):
 
 
 class CustomContractSuccessSpider(Spider):
-    name = 'custom_contract_success_spider'
+    name = "custom_contract_success_spider"
 
     def parse(self, response):
         """
@@ -187,7 +187,7 @@ class CustomContractSuccessSpider(Spider):
 
 
 class CustomContractFailSpider(Spider):
-    name = 'custom_contract_fail_spider'
+    name = "custom_contract_fail_spider"
 
     def parse(self, response):
         """
@@ -197,7 +197,7 @@ class CustomContractFailSpider(Spider):
 
 
 class InheritsTestSpider(TestSpider):
-    name = 'inherits_demo_spider'
+    name = "inherits_demo_spider"
 
 
 class ContractsManagerTest(unittest.TestCase):
@@ -234,7 +234,8 @@ class ContractsManagerTest(unittest.TestCase):
         self.assertEqual(len(contracts), 2)
         self.assertEqual(
             frozenset(type(x) for x in contracts),
-            frozenset([UrlContract, ReturnsContract]))
+            frozenset([UrlContract, ReturnsContract]),
+        )
 
         # returns request for valid method
         request = self.conman.from_method(spider.returns_request, self.results)
@@ -251,26 +252,40 @@ class ContractsManagerTest(unittest.TestCase):
         # extract contracts correctly
         contracts = self.conman.extract_contracts(spider.returns_request_cb_kwargs)
         self.assertEqual(len(contracts), 3)
-        self.assertEqual(frozenset(type(x) for x in contracts),
-                         frozenset([UrlContract, CallbackKeywordArgumentsContract, ReturnsContract]))
+        self.assertEqual(
+            frozenset(type(x) for x in contracts),
+            frozenset([UrlContract, CallbackKeywordArgumentsContract, ReturnsContract]),
+        )
 
         contracts = self.conman.extract_contracts(spider.returns_item_cb_kwargs)
         self.assertEqual(len(contracts), 3)
-        self.assertEqual(frozenset(type(x) for x in contracts),
-                         frozenset([UrlContract, CallbackKeywordArgumentsContract, ReturnsContract]))
+        self.assertEqual(
+            frozenset(type(x) for x in contracts),
+            frozenset([UrlContract, CallbackKeywordArgumentsContract, ReturnsContract]),
+        )
 
-        contracts = self.conman.extract_contracts(spider.returns_item_cb_kwargs_error_unexpected_keyword)
+        contracts = self.conman.extract_contracts(
+            spider.returns_item_cb_kwargs_error_unexpected_keyword
+        )
         self.assertEqual(len(contracts), 3)
-        self.assertEqual(frozenset(type(x) for x in contracts),
-                         frozenset([UrlContract, CallbackKeywordArgumentsContract, ReturnsContract]))
+        self.assertEqual(
+            frozenset(type(x) for x in contracts),
+            frozenset([UrlContract, CallbackKeywordArgumentsContract, ReturnsContract]),
+        )
 
-        contracts = self.conman.extract_contracts(spider.returns_item_cb_kwargs_error_missing_argument)
+        contracts = self.conman.extract_contracts(
+            spider.returns_item_cb_kwargs_error_missing_argument
+        )
         self.assertEqual(len(contracts), 2)
-        self.assertEqual(frozenset(type(x) for x in contracts),
-                         frozenset([UrlContract, ReturnsContract]))
+        self.assertEqual(
+            frozenset(type(x) for x in contracts),
+            frozenset([UrlContract, ReturnsContract]),
+        )
 
         # returns_request
-        request = self.conman.from_method(spider.returns_request_cb_kwargs, self.results)
+        request = self.conman.from_method(
+            spider.returns_request_cb_kwargs, self.results
+        )
         request.callback(response, **request.cb_kwargs)
         self.should_succeed()
 
@@ -280,12 +295,16 @@ class ContractsManagerTest(unittest.TestCase):
         self.should_succeed()
 
         # returns_item (error, callback doesn't take keyword arguments)
-        request = self.conman.from_method(spider.returns_item_cb_kwargs_error_unexpected_keyword, self.results)
+        request = self.conman.from_method(
+            spider.returns_item_cb_kwargs_error_unexpected_keyword, self.results
+        )
         request.callback(response, **request.cb_kwargs)
         self.should_error()
 
         # returns_item (error, contract doesn't provide keyword arguments)
-        request = self.conman.from_method(spider.returns_item_cb_kwargs_error_missing_argument, self.results)
+        request = self.conman.from_method(
+            spider.returns_item_cb_kwargs_error_missing_argument, self.results
+        )
         request.callback(response, **request.cb_kwargs)
         self.should_error()
 
@@ -343,10 +362,12 @@ class ContractsManagerTest(unittest.TestCase):
         self.should_fail()
 
         # scrapes_multiple_missing_fields
-        request = self.conman.from_method(spider.scrapes_multiple_missing_fields, self.results)
+        request = self.conman.from_method(
+            spider.scrapes_multiple_missing_fields, self.results
+        )
         request.callback(response)
         self.should_fail()
-        message = 'ContractFail: Missing fields: name, url'
+        message = "ContractFail: Missing fields: name, url"
         assert message in self.results.failures[-1][-1]
 
     def test_custom_contracts(self):
@@ -361,7 +382,7 @@ class ContractsManagerTest(unittest.TestCase):
         response = ResponseMock()
 
         try:
-            raise HttpError(response, 'Ignoring non-200 response')
+            raise HttpError(response, "Ignoring non-200 response")
         except HttpError:
             failure_mock = failure.Failure()
 
@@ -373,9 +394,8 @@ class ContractsManagerTest(unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_same_url(self):
-
         class TestSameUrlSpider(Spider):
-            name = 'test_same_url'
+            name = "test_same_url"
 
             def __init__(self, *args, **kwargs):
                 super().__init__(*args, **kwargs)
@@ -398,7 +418,7 @@ class ContractsManagerTest(unittest.TestCase):
             TestSameUrlSpider.parse_first.__doc__ = contract_doc
             TestSameUrlSpider.parse_second.__doc__ = contract_doc
 
-            crawler = CrawlerRunner().create_crawler(TestSameUrlSpider)
+            crawler = get_crawler(TestSameUrlSpider)
             yield crawler.crawl()
 
         self.assertEqual(crawler.spider.visited, 2)
@@ -406,7 +426,7 @@ class ContractsManagerTest(unittest.TestCase):
     def test_form_contract(self):
         spider = TestSpider()
         request = self.conman.from_method(spider.custom_form, self.results)
-        self.assertEqual(request.method, 'POST')
+        self.assertEqual(request.method, "POST")
         self.assertIsInstance(request, FormRequest)
 
     def test_inherited_contracts(self):
