@@ -10,16 +10,17 @@ module with the ``runserver`` argument::
     python test_engine.py runserver
 """
 
-import os
 import re
 import subprocess
 import sys
-import warnings
 from collections import defaultdict
+from dataclasses import dataclass
+from pathlib import Path
 from threading import Timer
 from urllib.parse import urlparse
 
 import attr
+import pytest
 from itemadapter import ItemAdapter
 from pydispatch import dispatcher
 from twisted.internet import defer, reactor
@@ -30,12 +31,11 @@ from scrapy import signals
 from scrapy.core.engine import ExecutionEngine
 from scrapy.exceptions import CloseSpider, ScrapyDeprecationWarning
 from scrapy.http import Request
-from scrapy.item import Item, Field
+from scrapy.item import Field, Item
 from scrapy.linkextractors import LinkExtractor
 from scrapy.spiders import Spider
 from scrapy.utils.signal import disconnect_all
 from scrapy.utils.test import get_crawler
-
 from tests import get_testdata, tests_datadir
 
 
@@ -52,6 +52,13 @@ class AttrsItem:
     price = attr.ib(default=0)
 
 
+@dataclass
+class DataClassItem:
+    name: str = ""
+    url: str = ""
+    price: int = 0
+
+
 class TestSpider(Spider):
     name = "scrapytest.org"
     allowed_domains = ["scrapytest.org", "localhost"]
@@ -73,11 +80,11 @@ class TestSpider(Spider):
         adapter = ItemAdapter(self.item_cls())
         m = self.name_re.search(response.text)
         if m:
-            adapter['name'] = m.group(1)
-        adapter['url'] = response.url
+            adapter["name"] = m.group(1)
+        adapter["url"] = response.url
         m = self.price_re.search(response.text)
         if m:
-            adapter['price'] = m.group(1)
+            adapter["price"] = m.group(1)
         return adapter.item
 
 
@@ -94,23 +101,14 @@ class AttrsItemsSpider(TestSpider):
     item_cls = AttrsItem
 
 
-try:
-    from dataclasses import make_dataclass
-except ImportError:
-    DataClassItemsSpider = None
-else:
-    TestDataClass = make_dataclass("TestDataClass", [("name", str), ("url", str), ("price", int)])
-
-    class DataClassItemsSpider(DictItemsSpider):  # type: ignore[no-redef]
-        def parse_item(self, response):
-            item = super().parse_item(response)
-            return TestDataClass(**item)
+class DataClassItemsSpider(TestSpider):
+    item_cls = DataClassItem
 
 
 class ItemZeroDivisionErrorSpider(TestSpider):
     custom_settings = {
         "ITEM_PIPELINES": {
-            "tests.pipelines.ProcessWithZeroDivisionErrorPipiline": 300,
+            "tests.pipelines.ProcessWithZeroDivisionErrorPipeline": 300,
         }
     }
 
@@ -128,8 +126,8 @@ class ChangeCloseReasonSpider(TestSpider):
 
 
 def start_test_site(debug=False):
-    root_dir = os.path.join(tests_datadir, "test_site")
-    r = static.File(root_dir)
+    root_dir = Path(tests_datadir, "test_site")
+    r = static.File(str(root_dir))
     r.putChild(b"redirect", util.Redirect(b"/redirected"))
     r.putChild(b"redirected", static.Data(b"Redirected here", "text/plain"))
     numbers = [str(x).encode("utf8") for x in range(2**18)]
@@ -137,8 +135,10 @@ def start_test_site(debug=False):
 
     port = reactor.listenTCP(0, server.Site(r), interface="127.0.0.1")
     if debug:
-        print(f"Test server running at http://localhost:{port.getHost().port}/ "
-              "- hit Ctrl-C to finish.")
+        print(
+            f"Test server running at http://localhost:{port.getHost().port}/ "
+            "- hit Ctrl-C to finish."
+        )
     return port
 
 
@@ -170,7 +170,7 @@ class CrawlerRun:
         ]
 
         for name, signal in vars(signals).items():
-            if not name.startswith('_'):
+            if not name.startswith("_"):
                 dispatcher.connect(self.record_signal, signal)
 
         self.crawler = get_crawler(self.spider_class)
@@ -180,8 +180,12 @@ class CrawlerRun:
         self.crawler.signals.connect(self.bytes_received, signals.bytes_received)
         self.crawler.signals.connect(self.request_scheduled, signals.request_scheduled)
         self.crawler.signals.connect(self.request_dropped, signals.request_dropped)
-        self.crawler.signals.connect(self.request_reached, signals.request_reached_downloader)
-        self.crawler.signals.connect(self.response_downloaded, signals.response_downloaded)
+        self.crawler.signals.connect(
+            self.request_reached, signals.request_reached_downloader
+        )
+        self.crawler.signals.connect(
+            self.response_downloaded, signals.response_downloaded
+        )
         self.crawler.crawl(start_urls=start_urls)
         self.spider = self.crawler.spider
 
@@ -190,9 +194,9 @@ class CrawlerRun:
         return self.deferred
 
     def stop(self):
-        self.port.stopListening()
+        self.port.stopListening()  # FIXME: wait for this Deferred
         for name, signal in vars(signals).items():
-            if not name.startswith('_'):
+            if not name.startswith("_"):
                 disconnect_all(signal)
         self.deferred.callback(None)
         return self.crawler.stop()
@@ -231,8 +235,8 @@ class CrawlerRun:
     def record_signal(self, *args, **kwargs):
         """Record a signal and its parameters"""
         signalargs = kwargs.copy()
-        sig = signalargs.pop('signal')
-        signalargs.pop('sender', None)
+        sig = signalargs.pop("signal")
+        signalargs.pop("sender", None)
         self.signals_caught[sig] = signalargs
 
 
@@ -240,133 +244,146 @@ class EngineTest(unittest.TestCase):
     @defer.inlineCallbacks
     def test_crawler(self):
 
-        for spider in (TestSpider, DictItemsSpider, AttrsItemsSpider, DataClassItemsSpider):
-            if spider is None:
-                continue
-            self.run = CrawlerRun(spider)
-            yield self.run.run()
-            self._assert_visited_urls()
-            self._assert_scheduled_requests(count=9)
-            self._assert_downloaded_responses(count=9)
-            self._assert_scraped_items()
-            self._assert_signals_caught()
-            self._assert_bytes_received()
+        for spider in (
+            TestSpider,
+            DictItemsSpider,
+            AttrsItemsSpider,
+            DataClassItemsSpider,
+        ):
+            run = CrawlerRun(spider)
+            yield run.run()
+            self._assert_visited_urls(run)
+            self._assert_scheduled_requests(run, count=9)
+            self._assert_downloaded_responses(run, count=9)
+            self._assert_scraped_items(run)
+            self._assert_signals_caught(run)
+            self._assert_bytes_received(run)
 
     @defer.inlineCallbacks
     def test_crawler_dupefilter(self):
-        self.run = CrawlerRun(TestDupeFilterSpider)
-        yield self.run.run()
-        self._assert_scheduled_requests(count=8)
-        self._assert_dropped_requests()
+        run = CrawlerRun(TestDupeFilterSpider)
+        yield run.run()
+        self._assert_scheduled_requests(run, count=8)
+        self._assert_dropped_requests(run)
 
     @defer.inlineCallbacks
     def test_crawler_itemerror(self):
-        self.run = CrawlerRun(ItemZeroDivisionErrorSpider)
-        yield self.run.run()
-        self._assert_items_error()
+        run = CrawlerRun(ItemZeroDivisionErrorSpider)
+        yield run.run()
+        self._assert_items_error(run)
 
     @defer.inlineCallbacks
     def test_crawler_change_close_reason_on_idle(self):
-        self.run = CrawlerRun(ChangeCloseReasonSpider)
-        yield self.run.run()
-        self.assertEqual({'spider': self.run.spider, 'reason': 'custom_reason'},
-                         self.run.signals_caught[signals.spider_closed])
+        run = CrawlerRun(ChangeCloseReasonSpider)
+        yield run.run()
+        self.assertEqual(
+            {"spider": run.spider, "reason": "custom_reason"},
+            run.signals_caught[signals.spider_closed],
+        )
 
-    def _assert_visited_urls(self):
-        must_be_visited = ["/", "/redirect", "/redirected",
-                           "/item1.html", "/item2.html", "/item999.html"]
-        urls_visited = {rp[0].url for rp in self.run.respplug}
-        urls_expected = {self.run.geturl(p) for p in must_be_visited}
-        assert urls_expected <= urls_visited, f"URLs not visited: {list(urls_expected - urls_visited)}"
+    def _assert_visited_urls(self, run: CrawlerRun):
+        must_be_visited = [
+            "/",
+            "/redirect",
+            "/redirected",
+            "/item1.html",
+            "/item2.html",
+            "/item999.html",
+        ]
+        urls_visited = {rp[0].url for rp in run.respplug}
+        urls_expected = {run.geturl(p) for p in must_be_visited}
+        assert (
+            urls_expected <= urls_visited
+        ), f"URLs not visited: {list(urls_expected - urls_visited)}"
 
-    def _assert_scheduled_requests(self, count=None):
-        self.assertEqual(count, len(self.run.reqplug))
+    def _assert_scheduled_requests(self, run: CrawlerRun, count=None):
+        self.assertEqual(count, len(run.reqplug))
 
-        paths_expected = ['/item999.html', '/item2.html', '/item1.html']
+        paths_expected = ["/item999.html", "/item2.html", "/item1.html"]
 
-        urls_requested = {rq[0].url for rq in self.run.reqplug}
-        urls_expected = {self.run.geturl(p) for p in paths_expected}
+        urls_requested = {rq[0].url for rq in run.reqplug}
+        urls_expected = {run.geturl(p) for p in paths_expected}
         assert urls_expected <= urls_requested
-        scheduled_requests_count = len(self.run.reqplug)
-        dropped_requests_count = len(self.run.reqdropped)
-        responses_count = len(self.run.respplug)
-        self.assertEqual(scheduled_requests_count,
-                         dropped_requests_count + responses_count)
-        self.assertEqual(len(self.run.reqreached),
-                         responses_count)
+        scheduled_requests_count = len(run.reqplug)
+        dropped_requests_count = len(run.reqdropped)
+        responses_count = len(run.respplug)
+        self.assertEqual(
+            scheduled_requests_count, dropped_requests_count + responses_count
+        )
+        self.assertEqual(len(run.reqreached), responses_count)
 
-    def _assert_dropped_requests(self):
-        self.assertEqual(len(self.run.reqdropped), 1)
+    def _assert_dropped_requests(self, run: CrawlerRun):
+        self.assertEqual(len(run.reqdropped), 1)
 
-    def _assert_downloaded_responses(self, count):
+    def _assert_downloaded_responses(self, run: CrawlerRun, count):
         # response tests
-        self.assertEqual(count, len(self.run.respplug))
-        self.assertEqual(count, len(self.run.reqreached))
+        self.assertEqual(count, len(run.respplug))
+        self.assertEqual(count, len(run.reqreached))
 
-        for response, _ in self.run.respplug:
-            if self.run.getpath(response.url) == '/item999.html':
+        for response, _ in run.respplug:
+            if run.getpath(response.url) == "/item999.html":
                 self.assertEqual(404, response.status)
-            if self.run.getpath(response.url) == '/redirect':
+            if run.getpath(response.url) == "/redirect":
                 self.assertEqual(302, response.status)
 
-    def _assert_items_error(self):
-        self.assertEqual(2, len(self.run.itemerror))
-        for item, response, spider, failure in self.run.itemerror:
+    def _assert_items_error(self, run: CrawlerRun):
+        self.assertEqual(2, len(run.itemerror))
+        for item, response, spider, failure in run.itemerror:
             self.assertEqual(failure.value.__class__, ZeroDivisionError)
-            self.assertEqual(spider, self.run.spider)
+            self.assertEqual(spider, run.spider)
 
-            self.assertEqual(item['url'], response.url)
-            if 'item1.html' in item['url']:
-                self.assertEqual('Item 1 name', item['name'])
-                self.assertEqual('100', item['price'])
-            if 'item2.html' in item['url']:
-                self.assertEqual('Item 2 name', item['name'])
-                self.assertEqual('200', item['price'])
+            self.assertEqual(item["url"], response.url)
+            if "item1.html" in item["url"]:
+                self.assertEqual("Item 1 name", item["name"])
+                self.assertEqual("100", item["price"])
+            if "item2.html" in item["url"]:
+                self.assertEqual("Item 2 name", item["name"])
+                self.assertEqual("200", item["price"])
 
-    def _assert_scraped_items(self):
-        self.assertEqual(2, len(self.run.itemresp))
-        for item, response in self.run.itemresp:
+    def _assert_scraped_items(self, run: CrawlerRun):
+        self.assertEqual(2, len(run.itemresp))
+        for item, response in run.itemresp:
             item = ItemAdapter(item)
-            self.assertEqual(item['url'], response.url)
-            if 'item1.html' in item['url']:
-                self.assertEqual('Item 1 name', item['name'])
-                self.assertEqual('100', item['price'])
-            if 'item2.html' in item['url']:
-                self.assertEqual('Item 2 name', item['name'])
-                self.assertEqual('200', item['price'])
+            self.assertEqual(item["url"], response.url)
+            if "item1.html" in item["url"]:
+                self.assertEqual("Item 1 name", item["name"])
+                self.assertEqual("100", item["price"])
+            if "item2.html" in item["url"]:
+                self.assertEqual("Item 2 name", item["name"])
+                self.assertEqual("200", item["price"])
 
-    def _assert_headers_received(self):
-        for headers in self.run.headers.values():
+    def _assert_headers_received(self, run: CrawlerRun):
+        for headers in run.headers.values():
             self.assertIn(b"Server", headers)
             self.assertIn(b"TwistedWeb", headers[b"Server"])
             self.assertIn(b"Date", headers)
             self.assertIn(b"Content-Type", headers)
 
-    def _assert_bytes_received(self):
-        self.assertEqual(9, len(self.run.bytes))
-        for request, data in self.run.bytes.items():
+    def _assert_bytes_received(self, run: CrawlerRun):
+        self.assertEqual(9, len(run.bytes))
+        for request, data in run.bytes.items():
             joined_data = b"".join(data)
-            if self.run.getpath(request.url) == "/":
+            if run.getpath(request.url) == "/":
                 self.assertEqual(joined_data, get_testdata("test_site", "index.html"))
-            elif self.run.getpath(request.url) == "/item1.html":
+            elif run.getpath(request.url) == "/item1.html":
                 self.assertEqual(joined_data, get_testdata("test_site", "item1.html"))
-            elif self.run.getpath(request.url) == "/item2.html":
+            elif run.getpath(request.url) == "/item2.html":
                 self.assertEqual(joined_data, get_testdata("test_site", "item2.html"))
-            elif self.run.getpath(request.url) == "/redirected":
+            elif run.getpath(request.url) == "/redirected":
                 self.assertEqual(joined_data, b"Redirected here")
-            elif self.run.getpath(request.url) == '/redirect':
+            elif run.getpath(request.url) == "/redirect":
                 self.assertEqual(
                     joined_data,
                     b"\n<html>\n"
                     b"    <head>\n"
-                    b"        <meta http-equiv=\"refresh\" content=\"0;URL=/redirected\">\n"
+                    b'        <meta http-equiv="refresh" content="0;URL=/redirected">\n'
                     b"    </head>\n"
-                    b"    <body bgcolor=\"#FFFFFF\" text=\"#000000\">\n"
-                    b"    <a href=\"/redirected\">click here</a>\n"
+                    b'    <body bgcolor="#FFFFFF" text="#000000">\n'
+                    b'    <a href="/redirected">click here</a>\n'
                     b"    </body>\n"
-                    b"</html>\n"
+                    b"</html>\n",
                 )
-            elif self.run.getpath(request.url) == "/tem999.html":
+            elif run.getpath(request.url) == "/tem999.html":
                 self.assertEqual(
                     joined_data,
                     b"\n<html>\n"
@@ -375,29 +392,33 @@ class EngineTest(unittest.TestCase):
                     b"    <h1>No Such Resource</h1>\n"
                     b"    <p>File not found.</p>\n"
                     b"  </body>\n"
-                    b"</html>\n"
+                    b"</html>\n",
                 )
-            elif self.run.getpath(request.url) == "/numbers":
+            elif run.getpath(request.url) == "/numbers":
                 # signal was fired multiple times
                 self.assertTrue(len(data) > 1)
                 # bytes were received in order
                 numbers = [str(x).encode("utf8") for x in range(2**18)]
                 self.assertEqual(joined_data, b"".join(numbers))
 
-    def _assert_signals_caught(self):
-        assert signals.engine_started in self.run.signals_caught
-        assert signals.engine_stopped in self.run.signals_caught
-        assert signals.spider_opened in self.run.signals_caught
-        assert signals.spider_idle in self.run.signals_caught
-        assert signals.spider_closed in self.run.signals_caught
-        assert signals.headers_received in self.run.signals_caught
+    def _assert_signals_caught(self, run: CrawlerRun):
+        assert signals.engine_started in run.signals_caught
+        assert signals.engine_stopped in run.signals_caught
+        assert signals.spider_opened in run.signals_caught
+        assert signals.spider_idle in run.signals_caught
+        assert signals.spider_closed in run.signals_caught
+        assert signals.headers_received in run.signals_caught
 
-        self.assertEqual({'spider': self.run.spider},
-                         self.run.signals_caught[signals.spider_opened])
-        self.assertEqual({'spider': self.run.spider},
-                         self.run.signals_caught[signals.spider_idle])
-        self.assertEqual({'spider': self.run.spider, 'reason': 'finished'},
-                         self.run.signals_caught[signals.spider_closed])
+        self.assertEqual(
+            {"spider": run.spider}, run.signals_caught[signals.spider_opened]
+        )
+        self.assertEqual(
+            {"spider": run.spider}, run.signals_caught[signals.spider_idle]
+        )
+        self.assertEqual(
+            {"spider": run.spider, "reason": "finished"},
+            run.signals_caught[signals.spider_closed],
+        )
 
     @defer.inlineCallbacks
     def test_close_downloader(self):
@@ -409,28 +430,33 @@ class EngineTest(unittest.TestCase):
         e = ExecutionEngine(get_crawler(TestSpider), lambda _: None)
         yield e.open_spider(TestSpider(), [])
         e.start()
-        yield self.assertFailure(e.start(), RuntimeError).addBoth(
-            lambda exc: self.assertEqual(str(exc), "Engine already running")
-        )
-        yield e.stop()
+        try:
+            yield self.assertFailure(e.start(), RuntimeError).addBoth(
+                lambda exc: self.assertEqual(str(exc), "Engine already running")
+            )
+        finally:
+            yield e.stop()
 
     @defer.inlineCallbacks
     def test_close_spiders_downloader(self):
-        with warnings.catch_warnings(record=True) as warning_list:
+        with pytest.warns(
+            ScrapyDeprecationWarning,
+            match="ExecutionEngine.open_spiders is deprecated, "
+            "please use ExecutionEngine.spider instead",
+        ):
             e = ExecutionEngine(get_crawler(TestSpider), lambda _: None)
             yield e.open_spider(TestSpider(), [])
             self.assertEqual(len(e.open_spiders), 1)
             yield e.close()
             self.assertEqual(len(e.open_spiders), 0)
-            self.assertEqual(warning_list[0].category, ScrapyDeprecationWarning)
-            self.assertEqual(
-                str(warning_list[0].message),
-                "ExecutionEngine.open_spiders is deprecated, please use ExecutionEngine.spider instead",
-            )
 
     @defer.inlineCallbacks
     def test_close_engine_spiders_downloader(self):
-        with warnings.catch_warnings(record=True) as warning_list:
+        with pytest.warns(
+            ScrapyDeprecationWarning,
+            match="ExecutionEngine.open_spiders is deprecated, "
+            "please use ExecutionEngine.spider instead",
+        ):
             e = ExecutionEngine(get_crawler(TestSpider), lambda _: None)
             yield e.open_spider(TestSpider(), [])
             e.start()
@@ -438,61 +464,54 @@ class EngineTest(unittest.TestCase):
             yield e.close()
             self.assertFalse(e.running)
             self.assertEqual(len(e.open_spiders), 0)
-            self.assertEqual(warning_list[0].category, ScrapyDeprecationWarning)
-            self.assertEqual(
-                str(warning_list[0].message),
-                "ExecutionEngine.open_spiders is deprecated, please use ExecutionEngine.spider instead",
-            )
 
     @defer.inlineCallbacks
     def test_crawl_deprecated_spider_arg(self):
-        with warnings.catch_warnings(record=True) as warning_list:
+        with pytest.warns(
+            ScrapyDeprecationWarning,
+            match="Passing a 'spider' argument to "
+            "ExecutionEngine.crawl is deprecated",
+        ):
             e = ExecutionEngine(get_crawler(TestSpider), lambda _: None)
             spider = TestSpider()
             yield e.open_spider(spider, [])
             e.start()
             e.crawl(Request("data:,"), spider)
             yield e.close()
-            self.assertEqual(warning_list[0].category, ScrapyDeprecationWarning)
-            self.assertEqual(
-                str(warning_list[0].message),
-                "Passing a 'spider' argument to ExecutionEngine.crawl is deprecated",
-            )
 
     @defer.inlineCallbacks
     def test_download_deprecated_spider_arg(self):
-        with warnings.catch_warnings(record=True) as warning_list:
+        with pytest.warns(
+            ScrapyDeprecationWarning,
+            match="Passing a 'spider' argument to "
+            "ExecutionEngine.download is deprecated",
+        ):
             e = ExecutionEngine(get_crawler(TestSpider), lambda _: None)
             spider = TestSpider()
             yield e.open_spider(spider, [])
             e.start()
             e.download(Request("data:,"), spider)
             yield e.close()
-            self.assertEqual(warning_list[0].category, ScrapyDeprecationWarning)
-            self.assertEqual(
-                str(warning_list[0].message),
-                "Passing a 'spider' argument to ExecutionEngine.download is deprecated",
-            )
 
     @defer.inlineCallbacks
     def test_deprecated_schedule(self):
-        with warnings.catch_warnings(record=True) as warning_list:
+        with pytest.warns(
+            ScrapyDeprecationWarning,
+            match="ExecutionEngine.schedule is deprecated, please use "
+            "ExecutionEngine.crawl or ExecutionEngine.download instead",
+        ):
             e = ExecutionEngine(get_crawler(TestSpider), lambda _: None)
             spider = TestSpider()
             yield e.open_spider(spider, [])
             e.start()
             e.schedule(Request("data:,"), spider)
             yield e.close()
-            self.assertEqual(warning_list[0].category, ScrapyDeprecationWarning)
-            self.assertEqual(
-                str(warning_list[0].message),
-                "ExecutionEngine.schedule is deprecated, please use "
-                "ExecutionEngine.crawl or ExecutionEngine.download instead",
-            )
 
     @defer.inlineCallbacks
     def test_deprecated_has_capacity(self):
-        with warnings.catch_warnings(record=True) as warning_list:
+        with pytest.warns(
+            ScrapyDeprecationWarning, match="ExecutionEngine.has_capacity is deprecated"
+        ):
             e = ExecutionEngine(get_crawler(TestSpider), lambda _: None)
             self.assertTrue(e.has_capacity())
             spider = TestSpider()
@@ -501,20 +520,18 @@ class EngineTest(unittest.TestCase):
             e.start()
             yield e.close()
             self.assertTrue(e.has_capacity())
-            self.assertEqual(warning_list[0].category, ScrapyDeprecationWarning)
-            self.assertEqual(str(warning_list[0].message), "ExecutionEngine.has_capacity is deprecated")
 
     def test_short_timeout(self):
         args = (
             sys.executable,
-            '-m',
-            'scrapy.cmdline',
-            'fetch',
-            '-s',
-            'CLOSESPIDER_TIMEOUT=0.001',
-            '-s',
-            'LOG_LEVEL=DEBUG',
-            'http://toscrape.com',
+            "-m",
+            "scrapy.cmdline",
+            "fetch",
+            "-s",
+            "CLOSESPIDER_TIMEOUT=0.001",
+            "-s",
+            "LOG_LEVEL=DEBUG",
+            "http://toscrape.com",
         )
         p = subprocess.Popen(
             args,
@@ -524,7 +541,7 @@ class EngineTest(unittest.TestCase):
         def kill_proc():
             p.kill()
             p.communicate()
-            assert False, 'Command took too much time to complete'
+            assert False, "Command took too much time to complete"
 
         timer = Timer(15, kill_proc)
         try:
@@ -533,10 +550,10 @@ class EngineTest(unittest.TestCase):
         finally:
             timer.cancel()
 
-        self.assertNotIn(b'Traceback', stderr)
+        self.assertNotIn(b"Traceback", stderr)
 
 
 if __name__ == "__main__":
-    if len(sys.argv) > 1 and sys.argv[1] == 'runserver':
+    if len(sys.argv) > 1 and sys.argv[1] == "runserver":
         start_test_site(debug=True)
         reactor.run()
