@@ -1,7 +1,7 @@
-from twisted.trial import unittest
 from twisted.conch.telnet import ITelnetProtocol
 from twisted.cred import credentials
 from twisted.internet import defer
+from twisted.trial import unittest
 
 from scrapy.extensions.telnet import TelnetConsole
 from scrapy.utils.test import get_crawler
@@ -24,7 +24,7 @@ class TelnetExtensionTest(unittest.TestCase):
     @defer.inlineCallbacks
     def test_bad_credentials(self):
         console, portal = self._get_console_and_portal()
-        creds = credentials.UsernamePassword(b'username', b'password')
+        creds = credentials.UsernamePassword(b"username", b"password")
         d = portal.login(creds, None, ITelnetProtocol)
         yield self.assertFailure(d, ValueError)
         console.stop_listening()
@@ -33,8 +33,7 @@ class TelnetExtensionTest(unittest.TestCase):
     def test_good_credentials(self):
         console, portal = self._get_console_and_portal()
         creds = credentials.UsernamePassword(
-            console.username.encode('utf8'),
-            console.password.encode('utf8')
+            console.username.encode("utf8"), console.password.encode("utf8")
         )
         d = portal.login(creds, None, ITelnetProtocol)
         yield d
@@ -43,11 +42,11 @@ class TelnetExtensionTest(unittest.TestCase):
     @defer.inlineCallbacks
     def test_custom_credentials(self):
         settings = {
-            'TELNETCONSOLE_USERNAME': 'user',
-            'TELNETCONSOLE_PASSWORD': 'pass',
+            "TELNETCONSOLE_USERNAME": "user",
+            "TELNETCONSOLE_PASSWORD": "pass",
         }
         console, portal = self._get_console_and_portal(settings=settings)
-        creds = credentials.UsernamePassword(b'user', b'pass')
+        creds = credentials.UsernamePassword(b"user", b"pass")
         d = portal.login(creds, None, ITelnetProtocol)
         yield d
         console.stop_listening()
