@@ -1,12 +1,11 @@
 from twisted.trial import unittest
 
-from scrapy.settings import Settings
 from scrapy.exceptions import NotConfigured
 from scrapy.middleware import MiddlewareManager
+from scrapy.settings import Settings
 
 
 class M1:
-
     def open_spider(self, spider):
         pass
 
@@ -18,7 +17,6 @@ class M1:
 
 
 class M2:
-
     def open_spider(self, spider):
         pass
 
@@ -29,13 +27,11 @@ class M2:
 
 
 class M3:
-
     def process(self, response, request, spider):
         pass
 
 
 class MOff:
-
     def open_spider(self, spider):
         pass
 
@@ -47,37 +43,39 @@ class MOff:
 
 
 class TestMiddlewareManager(MiddlewareManager):
-
     @classmethod
     def _get_mwlist_from_settings(cls, settings):
         return [M1, MOff, M3]
 
     def _add_middleware(self, mw):
         super()._add_middleware(mw)
-        if hasattr(mw, 'process'):
-            self.methods['process'].append(mw.process)
+        if hasattr(mw, "process"):
+            self.methods["process"].append(mw.process)
 
 
 class MiddlewareManagerTest(unittest.TestCase):
-
     def test_init(self):
         m1, m2, m3 = M1(), M2(), M3()
         mwman = TestMiddlewareManager(m1, m2, m3)
-        self.assertEqual(list(mwman.methods['open_spider']), [m1.open_spider, m2.open_spider])
-        self.assertEqual(list(mwman.methods['close_spider']), [m2.close_spider, m1.close_spider])
-        self.assertEqual(list(mwman.methods['process']), [m1.process, m3.process])
+        self.assertEqual(
+            list(mwman.methods["open_spider"]), [m1.open_spider, m2.open_spider]
+        )
+        self.assertEqual(
+            list(mwman.methods["close_spider"]), [m2.close_spider, m1.close_spider]
+        )
+        self.assertEqual(list(mwman.methods["process"]), [m1.process, m3.process])
 
     def test_methods(self):
         mwman = TestMiddlewareManager(M1(), M2(), M3())
         self.assertEqual(
-            [x.__self__.__class__ for x in mwman.methods['open_spider']],
-            [M1, M2])
+            [x.__self__.__class__ for x in mwman.methods["open_spider"]], [M1, M2]
+        )
         self.assertEqual(
-            [x.__self__.__class__ for x in mwman.methods['close_spider']],
-            [M2, M1])
+            [x.__self__.__class__ for x in mwman.methods["close_spider"]], [M2, M1]
+        )
         self.assertEqual(
-            [x.__self__.__class__ for x in mwman.methods['process']],
-            [M1, M3])
+            [x.__self__.__class__ for x in mwman.methods["process"]], [M1, M3]
+        )
 
     def test_enabled(self):
         m1, m2, m3 = M1(), M2(), M3()
