@@ -1,19 +1,19 @@
-import os
 import shutil
+from pathlib import Path
 
 from testfixtures import LogCapture
 from twisted.internet import defer
 from twisted.trial.unittest import TestCase
 from w3lib.url import add_or_replace_parameter
 
-from scrapy.crawler import CrawlerRunner
 from scrapy import signals
+from scrapy.crawler import CrawlerRunner
 from tests.mockserver import MockServer
 from tests.spiders import SimpleSpider
 
 
 class MediaDownloadSpider(SimpleSpider):
-    name = 'mediadownload'
+    name = "mediadownload"
 
     def _process_url(self, url):
         return url
@@ -27,45 +27,50 @@ class MediaDownloadSpider(SimpleSpider):
                 self._process_url(response.urljoin(href))
                 for href in response.xpath(
                     '//table[thead/tr/th="Filename"]/tbody//a/@href'
-                ).getall()],
+                ).getall()
+            ],
         }
         yield item
 
 
 class BrokenLinksMediaDownloadSpider(MediaDownloadSpider):
-    name = 'brokenmedia'
+    name = "brokenmedia"
 
     def _process_url(self, url):
-        return url + '.foo'
+        return url + ".foo"
 
 
 class RedirectedMediaDownloadSpider(MediaDownloadSpider):
-    name = 'redirectedmedia'
+    name = "redirectedmedia"
 
     def _process_url(self, url):
-        return add_or_replace_parameter(self.mockserver.url('/redirect-to'), 'goto', url)
+        return add_or_replace_parameter(
+            self.mockserver.url("/redirect-to"), "goto", url
+        )
 
 
 class FileDownloadCrawlTestCase(TestCase):
-    pipeline_class = 'scrapy.pipelines.files.FilesPipeline'
-    store_setting_key = 'FILES_STORE'
-    media_key = 'files'
-    media_urls_key = 'file_urls'
+    pipeline_class = "scrapy.pipelines.files.FilesPipeline"
+    store_setting_key = "FILES_STORE"
+    media_key = "files"
+    media_urls_key = "file_urls"
     expected_checksums = {
-        '5547178b89448faf0015a13f904c936e',
-        'c2281c83670e31d8aaab7cb642b824db',
-        'ed3f6538dc15d4d9179dae57319edc5f'}
+        "5547178b89448faf0015a13f904c936e",
+        "c2281c83670e31d8aaab7cb642b824db",
+        "ed3f6538dc15d4d9179dae57319edc5f",
+    }
 
     def setUp(self):
         self.mockserver = MockServer()
         self.mockserver.__enter__()
 
         # prepare a directory for storing files
-        self.tmpmediastore = self.mktemp()
-        os.mkdir(self.tmpmediastore)
+        self.tmpmediastore = Path(self.mktemp())
+        self.tmpmediastore.mkdir()
         self.settings = {
-            'ITEM_PIPELINES': {self.pipeline_class: 1},
-            self.store_setting_key: self.tmpmediastore,
+            "REQUEST_FINGERPRINTER_IMPLEMENTATION": "2.7",
+            "ITEM_PIPELINES": {self.pipeline_class: 1},
+            self.store_setting_key: str(self.tmpmediastore),
         }
         self.runner = CrawlerRunner(self.settings)
         self.items = []
@@ -78,8 +83,10 @@ class FileDownloadCrawlTestCase(TestCase):
     def _on_item_scraped(self, item):
         self.items.append(item)
 
-    def _create_crawler(self, spider_class, **kwargs):
-        crawler = self.runner.create_crawler(spider_class, **kwargs)
+    def _create_crawler(self, spider_class, runner=None, **kwargs):
+        if runner is None:
+            runner = self.runner
+        crawler = runner.create_crawler(spider_class, **kwargs)
         crawler.signals.connect(self._on_item_scraped, signals.item_scraped)
         return crawler
 
@@ -88,29 +95,25 @@ class FileDownloadCrawlTestCase(TestCase):
         self.assertIn(self.media_key, items[0])
 
         # check that logs show the expected number of successful file downloads
-        file_dl_success = 'File (downloaded): Downloaded file from'
+        file_dl_success = "File (downloaded): Downloaded file from"
         self.assertEqual(logs.count(file_dl_success), 3)
 
         # check that the images/files status is `downloaded`
         for item in items:
             for i in item[self.media_key]:
-                self.assertEqual(i['status'], 'downloaded')
+                self.assertEqual(i["status"], "downloaded")
 
         # check that the images/files checksums are what we know they should be
         if self.expected_checksums is not None:
             checksums = set(
-                i['checksum']
-                for item in items
-                for i in item[self.media_key]
+                i["checksum"] for item in items for i in item[self.media_key]
             )
             self.assertEqual(checksums, self.expected_checksums)
 
         # check that the image files where actually written to the media store
         for item in items:
             for i in item[self.media_key]:
-                self.assertTrue(
-                    os.path.exists(
-                        os.path.join(self.tmpmediastore, i['path'])))
+                self.assertTrue((self.tmpmediastore / i["path"]).exists())
 
     def _assert_files_download_failure(self, crawler, items, code, logs):
 
@@ -120,17 +123,23 @@ class FileDownloadCrawlTestCase(TestCase):
         self.assertFalse(items[0][self.media_key])
 
         # check that there was 1 successful fetch and 3 other responses with non-200 code
-        self.assertEqual(crawler.stats.get_value('downloader/request_method_count/GET'), 4)
-        self.assertEqual(crawler.stats.get_value('downloader/response_count'), 4)
-        self.assertEqual(crawler.stats.get_value('downloader/response_status_count/200'), 1)
-        self.assertEqual(crawler.stats.get_value(f'downloader/response_status_count/{code}'), 3)
+        self.assertEqual(
+            crawler.stats.get_value("downloader/request_method_count/GET"), 4
+        )
+        self.assertEqual(crawler.stats.get_value("downloader/response_count"), 4)
+        self.assertEqual(
+            crawler.stats.get_value("downloader/response_status_count/200"), 1
+        )
+        self.assertEqual(
+            crawler.stats.get_value(f"downloader/response_status_count/{code}"), 3
+        )
 
         # check that logs do show the failure on the file downloads
-        file_dl_failure = f'File (code: {code}): Error downloading file from'
+        file_dl_failure = f"File (code: {code}): Error downloading file from"
         self.assertEqual(logs.count(file_dl_failure), 3)
 
         # check that no files were written to the media store
-        self.assertEqual(os.listdir(self.tmpmediastore), [])
+        self.assertEqual([x for x in self.tmpmediastore.iterdir()], [])
 
     @defer.inlineCallbacks
     def test_download_media(self):
@@ -139,7 +148,8 @@ class FileDownloadCrawlTestCase(TestCase):
             yield crawler.crawl(
                 self.mockserver.url("/files/images/"),
                 media_key=self.media_key,
-                media_urls_key=self.media_urls_key)
+                media_urls_key=self.media_urls_key,
+            )
         self._assert_files_downloaded(self.items, str(log))
 
     @defer.inlineCallbacks
@@ -149,7 +159,8 @@ class FileDownloadCrawlTestCase(TestCase):
             yield crawler.crawl(
                 self.mockserver.url("/files/images/"),
                 media_key=self.media_key,
-                media_urls_key=self.media_urls_key)
+                media_urls_key=self.media_urls_key,
+            )
         self._assert_files_download_failure(crawler, self.items, 404, str(log))
 
     @defer.inlineCallbacks
@@ -160,30 +171,35 @@ class FileDownloadCrawlTestCase(TestCase):
                 self.mockserver.url("/files/images/"),
                 media_key=self.media_key,
                 media_urls_key=self.media_urls_key,
-                mockserver=self.mockserver)
+                mockserver=self.mockserver,
+            )
         self._assert_files_download_failure(crawler, self.items, 302, str(log))
 
     @defer.inlineCallbacks
     def test_download_media_redirected_allowed(self):
         settings = dict(self.settings)
-        settings.update({'MEDIA_ALLOW_REDIRECTS': True})
-        self.runner = CrawlerRunner(settings)
-
-        crawler = self._create_crawler(RedirectedMediaDownloadSpider)
+        settings.update({"MEDIA_ALLOW_REDIRECTS": True})
+        runner = CrawlerRunner(settings)
+        crawler = self._create_crawler(RedirectedMediaDownloadSpider, runner=runner)
         with LogCapture() as log:
             yield crawler.crawl(
                 self.mockserver.url("/files/images/"),
                 media_key=self.media_key,
                 media_urls_key=self.media_urls_key,
-                mockserver=self.mockserver)
+                mockserver=self.mockserver,
+            )
         self._assert_files_downloaded(self.items, str(log))
-        self.assertEqual(crawler.stats.get_value('downloader/response_status_count/302'), 3)
+        self.assertEqual(
+            crawler.stats.get_value("downloader/response_status_count/302"), 3
+        )
 
 
 try:
     from PIL import Image  # noqa: imported just to check for the import error
 except ImportError:
-    skip_pillow = 'Missing Python Imaging Library, install https://pypi.python.org/pypi/Pillow'
+    skip_pillow = (
+        "Missing Python Imaging Library, install https://pypi.python.org/pypi/Pillow"
+    )
 else:
     skip_pillow = None
 
@@ -192,10 +208,10 @@ class ImageDownloadCrawlTestCase(FileDownloadCrawlTestCase):
 
     skip = skip_pillow
 
-    pipeline_class = 'scrapy.pipelines.images.ImagesPipeline'
-    store_setting_key = 'IMAGES_STORE'
-    media_key = 'images'
-    media_urls_key = 'image_urls'
+    pipeline_class = "scrapy.pipelines.images.ImagesPipeline"
+    store_setting_key = "IMAGES_STORE"
+    media_key = "images"
+    media_urls_key = "image_urls"
 
     # somehow checksums for images are different for Python 3.3
     expected_checksums = None
