@@ -1,11 +1,13 @@
+import dataclasses
 import os
 import random
 import time
 from datetime import datetime
 from io import BytesIO
+from pathlib import Path
 from shutil import rmtree
 from tempfile import mkdtemp
-from unittest import mock, skipIf
+from unittest import mock
 from urllib.parse import urlparse
 
 import attr
@@ -25,29 +27,21 @@ from scrapy.pipelines.files import (
 from scrapy.settings import Settings
 from scrapy.utils.test import (
     assert_gcs_environ,
+    get_crawler,
     get_ftp_content_and_delete,
     get_gcs_content_and_delete,
     skip_if_no_boto,
 )
 
-
-try:
-    from dataclasses import make_dataclass, field as dataclass_field
-except ImportError:
-    make_dataclass = None
-    dataclass_field = None
-
-
-def _mocked_download_func(request, info):
-    response = request.meta.get('response')
-    return response() if callable(response) else response
+from .test_pipeline_media import _mocked_download_func
 
 
 class FilesPipelineTestCase(unittest.TestCase):
-
     def setUp(self):
         self.tempdir = mkdtemp()
-        self.pipeline = FilesPipeline.from_settings(Settings({'FILES_STORE': self.tempdir}))
+        settings_dict = {"FILES_STORE": self.tempdir}
+        crawler = get_crawler(spidercls=None, settings_dict=settings_dict)
+        self.pipeline = FilesPipeline.from_crawler(crawler)
         self.pipeline.download_func = _mocked_download_func
         self.pipeline.open_spider(None)
 
@@ -58,41 +52,70 @@ class FilesPipelineTestCase(unittest.TestCase):
         file_path = self.pipeline.file_path
         self.assertEqual(
             file_path(Request("https://dev.mydeco.com/mydeco.pdf")),
-            'full/c9b564df929f4bc635bdd19fde4f3d4847c757c5.pdf')
+            "full/c9b564df929f4bc635bdd19fde4f3d4847c757c5.pdf",
+        )
         self.assertEqual(
-            file_path(Request("http://www.maddiebrown.co.uk///catalogue-items//image_54642_12175_95307.txt")),
-            'full/4ce274dd83db0368bafd7e406f382ae088e39219.txt')
+            file_path(
+                Request(
+                    "http://www.maddiebrown.co.uk///catalogue-items//image_54642_12175_95307.txt"
+                )
+            ),
+            "full/4ce274dd83db0368bafd7e406f382ae088e39219.txt",
+        )
         self.assertEqual(
-            file_path(Request("https://dev.mydeco.com/two/dirs/with%20spaces%2Bsigns.doc")),
-            'full/94ccc495a17b9ac5d40e3eabf3afcb8c2c9b9e1a.doc')
+            file_path(
+                Request("https://dev.mydeco.com/two/dirs/with%20spaces%2Bsigns.doc")
+            ),
+            "full/94ccc495a17b9ac5d40e3eabf3afcb8c2c9b9e1a.doc",
+        )
         self.assertEqual(
-            file_path(Request("http://www.dfsonline.co.uk/get_prod_image.php?img=status_0907_mdm.jpg")),
-            'full/4507be485f38b0da8a0be9eb2e1dfab8a19223f2.jpg')
+            file_path(
+                Request(
+                    "http://www.dfsonline.co.uk/get_prod_image.php?img=status_0907_mdm.jpg"
+                )
+            ),
+            "full/4507be485f38b0da8a0be9eb2e1dfab8a19223f2.jpg",
+        )
         self.assertEqual(
             file_path(Request("http://www.dorma.co.uk/images/product_details/2532/")),
-            'full/97ee6f8a46cbbb418ea91502fd24176865cf39b2')
+            "full/97ee6f8a46cbbb418ea91502fd24176865cf39b2",
+        )
         self.assertEqual(
             file_path(Request("http://www.dorma.co.uk/images/product_details/2532")),
-            'full/244e0dd7d96a3b7b01f54eded250c9e272577aa1')
+            "full/244e0dd7d96a3b7b01f54eded250c9e272577aa1",
+        )
         self.assertEqual(
-            file_path(Request("http://www.dorma.co.uk/images/product_details/2532"),
-                      response=Response("http://www.dorma.co.uk/images/product_details/2532"),
-                      info=object()),
-            'full/244e0dd7d96a3b7b01f54eded250c9e272577aa1')
+            file_path(
+                Request("http://www.dorma.co.uk/images/product_details/2532"),
+                response=Response("http://www.dorma.co.uk/images/product_details/2532"),
+                info=object(),
+            ),
+            "full/244e0dd7d96a3b7b01f54eded250c9e272577aa1",
+        )
         self.assertEqual(
-            file_path(Request("http://www.dfsonline.co.uk/get_prod_image.php?img=status_0907_mdm.jpg.bohaha")),
-            'full/76c00cef2ef669ae65052661f68d451162829507')
+            file_path(
+                Request(
+                    "http://www.dfsonline.co.uk/get_prod_image.php?img=status_0907_mdm.jpg.bohaha"
+                )
+            ),
+            "full/76c00cef2ef669ae65052661f68d451162829507",
+        )
         self.assertEqual(
-            file_path(Request("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAR0AAACxCAMAAADOHZloAAACClBMVEX/\
-                                    //+F0tzCwMK76ZKQ21AMqr7oAAC96JvD5aWM2kvZ78J0N7fmAAC46Y4Ap7y")),
-            'full/178059cbeba2e34120a67f2dc1afc3ecc09b61cb.png')
+            file_path(
+                Request(
+                    "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAR0AAACxCAMAAADOHZloAAACClBMVEX/\
+                                    //+F0tzCwMK76ZKQ21AMqr7oAAC96JvD5aWM2kvZ78J0N7fmAAC46Y4Ap7y"
+                )
+            ),
+            "full/178059cbeba2e34120a67f2dc1afc3ecc09b61cb.png",
+        )
 
     def test_fs_store(self):
         assert isinstance(self.pipeline.store, FSFilesStore)
         self.assertEqual(self.pipeline.store.basedir, self.tempdir)
 
-        path = 'some/image/key.jpg'
-        fullpath = os.path.join(self.tempdir, 'some', 'image', 'key.jpg')
+        path = "some/image/key.jpg"
+        fullpath = Path(self.tempdir, "some", "image", "key.jpg")
         self.assertEqual(self.pipeline.store._get_filesystem_path(path), fullpath)
 
     @defer.inlineCallbacks
@@ -100,18 +123,24 @@ class FilesPipelineTestCase(unittest.TestCase):
         item_url = "http://example.com/file.pdf"
         item = _create_item_with_files(item_url)
         patchers = [
-            mock.patch.object(FilesPipeline, 'inc_stats', return_value=True),
-            mock.patch.object(FSFilesStore, 'stat_file', return_value={
-                'checksum': 'abc', 'last_modified': time.time()}),
-            mock.patch.object(FilesPipeline, 'get_media_requests',
-                              return_value=[_prepare_request_object(item_url)])
+            mock.patch.object(FilesPipeline, "inc_stats", return_value=True),
+            mock.patch.object(
+                FSFilesStore,
+                "stat_file",
+                return_value={"checksum": "abc", "last_modified": time.time()},
+            ),
+            mock.patch.object(
+                FilesPipeline,
+                "get_media_requests",
+                return_value=[_prepare_request_object(item_url)],
+            ),
         ]
         for p in patchers:
             p.start()
 
         result = yield self.pipeline.process_item(item, None)
-        self.assertEqual(result['files'][0]['checksum'], 'abc')
-        self.assertEqual(result['files'][0]['status'], 'uptodate')
+        self.assertEqual(result["files"][0]["checksum"], "abc")
+        self.assertEqual(result["files"][0]["status"], "uptodate")
 
         for p in patchers:
             p.stop()
@@ -121,19 +150,28 @@ class FilesPipelineTestCase(unittest.TestCase):
         item_url = "http://example.com/file2.pdf"
         item = _create_item_with_files(item_url)
         patchers = [
-            mock.patch.object(FSFilesStore, 'stat_file', return_value={
-                'checksum': 'abc',
-                'last_modified': time.time() - (self.pipeline.expires * 60 * 60 * 24 * 2)}),
-            mock.patch.object(FilesPipeline, 'get_media_requests',
-                              return_value=[_prepare_request_object(item_url)]),
-            mock.patch.object(FilesPipeline, 'inc_stats', return_value=True)
+            mock.patch.object(
+                FSFilesStore,
+                "stat_file",
+                return_value={
+                    "checksum": "abc",
+                    "last_modified": time.time()
+                    - (self.pipeline.expires * 60 * 60 * 24 * 2),
+                },
+            ),
+            mock.patch.object(
+                FilesPipeline,
+                "get_media_requests",
+                return_value=[_prepare_request_object(item_url)],
+            ),
+            mock.patch.object(FilesPipeline, "inc_stats", return_value=True),
         ]
         for p in patchers:
             p.start()
 
         result = yield self.pipeline.process_item(item, None)
-        self.assertNotEqual(result['files'][0]['checksum'], 'abc')
-        self.assertEqual(result['files'][0]['status'], 'downloaded')
+        self.assertNotEqual(result["files"][0]["checksum"], "abc")
+        self.assertEqual(result["files"][0]["status"], "downloaded")
 
         for p in patchers:
             p.stop()
@@ -143,19 +181,28 @@ class FilesPipelineTestCase(unittest.TestCase):
         item_url = "http://example.com/file3.pdf"
         item = _create_item_with_files(item_url)
         patchers = [
-            mock.patch.object(FilesPipeline, 'inc_stats', return_value=True),
-            mock.patch.object(FSFilesStore, 'stat_file', return_value={
-                'checksum': 'abc',
-                'last_modified': time.time() - (self.pipeline.expires * 60 * 60 * 24 * 2)}),
-            mock.patch.object(FilesPipeline, 'get_media_requests',
-                              return_value=[_prepare_request_object(item_url, flags=['cached'])])
+            mock.patch.object(FilesPipeline, "inc_stats", return_value=True),
+            mock.patch.object(
+                FSFilesStore,
+                "stat_file",
+                return_value={
+                    "checksum": "abc",
+                    "last_modified": time.time()
+                    - (self.pipeline.expires * 60 * 60 * 24 * 2),
+                },
+            ),
+            mock.patch.object(
+                FilesPipeline,
+                "get_media_requests",
+                return_value=[_prepare_request_object(item_url, flags=["cached"])],
+            ),
         ]
         for p in patchers:
             p.start()
 
         result = yield self.pipeline.process_item(item, None)
-        self.assertNotEqual(result['files'][0]['checksum'], 'abc')
-        self.assertEqual(result['files'][0]['status'], 'cached')
+        self.assertNotEqual(result["files"][0]["checksum"], "abc")
+        self.assertEqual(result["files"][0]["status"], "cached")
 
         for p in patchers:
             p.stop()
@@ -164,48 +211,58 @@ class FilesPipelineTestCase(unittest.TestCase):
         """
         Custom file path based on item data, overriding default implementation
         """
+
         class CustomFilesPipeline(FilesPipeline):
             def file_path(self, request, response=None, info=None, item=None):
                 return f'full/{item.get("path")}'
 
-        file_path = CustomFilesPipeline.from_settings(Settings({'FILES_STORE': self.tempdir})).file_path
-        item = dict(path='path-to-store-file')
+        file_path = CustomFilesPipeline.from_settings(
+            Settings({"FILES_STORE": self.tempdir})
+        ).file_path
+        item = dict(path="path-to-store-file")
         request = Request("http://example.com")
-        self.assertEqual(file_path(request, item=item), 'full/path-to-store-file')
+        self.assertEqual(file_path(request, item=item), "full/path-to-store-file")
 
 
 class FilesPipelineTestCaseFieldsMixin:
-
     def test_item_fields_default(self):
-        url = 'http://www.example.com/files/1.txt'
-        item = self.item_class(name='item1', file_urls=[url])
-        pipeline = FilesPipeline.from_settings(Settings({'FILES_STORE': 's3://example/files/'}))
+        url = "http://www.example.com/files/1.txt"
+        item = self.item_class(name="item1", file_urls=[url])
+        pipeline = FilesPipeline.from_settings(
+            Settings({"FILES_STORE": "s3://example/files/"})
+        )
         requests = list(pipeline.get_media_requests(item, None))
         self.assertEqual(requests[0].url, url)
-        results = [(True, {'url': url})]
+        results = [(True, {"url": url})]
         item = pipeline.item_completed(results, item, None)
         files = ItemAdapter(item).get("files")
         self.assertEqual(files, [results[0][1]])
         self.assertIsInstance(item, self.item_class)
 
     def test_item_fields_override_settings(self):
-        url = 'http://www.example.com/files/1.txt'
-        item = self.item_class(name='item1', custom_file_urls=[url])
-        pipeline = FilesPipeline.from_settings(Settings({
-            'FILES_STORE': 's3://example/files/',
-            'FILES_URLS_FIELD': 'custom_file_urls',
-            'FILES_RESULT_FIELD': 'custom_files'
-        }))
+        url = "http://www.example.com/files/1.txt"
+        item = self.item_class(name="item1", custom_file_urls=[url])
+        pipeline = FilesPipeline.from_settings(
+            Settings(
+                {
+                    "FILES_STORE": "s3://example/files/",
+                    "FILES_URLS_FIELD": "custom_file_urls",
+                    "FILES_RESULT_FIELD": "custom_files",
+                }
+            )
+        )
         requests = list(pipeline.get_media_requests(item, None))
         self.assertEqual(requests[0].url, url)
-        results = [(True, {'url': url})]
+        results = [(True, {"url": url})]
         item = pipeline.item_completed(results, item, None)
         custom_files = ItemAdapter(item).get("custom_files")
         self.assertEqual(custom_files, [results[0][1]])
         self.assertIsInstance(item, self.item_class)
 
 
-class FilesPipelineTestCaseFieldsDict(FilesPipelineTestCaseFieldsMixin, unittest.TestCase):
+class FilesPipelineTestCaseFieldsDict(
+    FilesPipelineTestCaseFieldsMixin, unittest.TestCase
+):
     item_class = dict
 
 
@@ -219,28 +276,27 @@ class FilesPipelineTestItem(Item):
     custom_files = Field()
 
 
-class FilesPipelineTestCaseFieldsItem(FilesPipelineTestCaseFieldsMixin, unittest.TestCase):
+class FilesPipelineTestCaseFieldsItem(
+    FilesPipelineTestCaseFieldsMixin, unittest.TestCase
+):
     item_class = FilesPipelineTestItem
 
 
-@skipIf(not make_dataclass, "dataclasses module is not available")
-class FilesPipelineTestCaseFieldsDataClass(FilesPipelineTestCaseFieldsMixin, unittest.TestCase):
+@dataclasses.dataclass
+class FilesPipelineTestDataClass:
+    name: str
+    # default fields
+    file_urls: list = dataclasses.field(default_factory=list)
+    files: list = dataclasses.field(default_factory=list)
+    # overridden fields
+    custom_file_urls: list = dataclasses.field(default_factory=list)
+    custom_files: list = dataclasses.field(default_factory=list)
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        if make_dataclass:
-            self.item_class = make_dataclass(
-                "FilesPipelineTestDataClass",
-                [
-                    ("name", str),
-                    # default fields
-                    ("file_urls", list, dataclass_field(default_factory=list)),
-                    ("files", list, dataclass_field(default_factory=list)),
-                    # overridden fields
-                    ("custom_file_urls", list, dataclass_field(default_factory=list)),
-                    ("custom_files", list, dataclass_field(default_factory=list)),
-                ],
-            )
+
+class FilesPipelineTestCaseFieldsDataClass(
+    FilesPipelineTestCaseFieldsMixin, unittest.TestCase
+):
+    item_class = FilesPipelineTestDataClass
 
 
 @attr.s
@@ -254,7 +310,9 @@ class FilesPipelineTestAttrsItem:
     custom_files = attr.ib(default=lambda: [])
 
 
-class FilesPipelineTestCaseFieldsAttrsItem(FilesPipelineTestCaseFieldsMixin, unittest.TestCase):
+class FilesPipelineTestCaseFieldsAttrsItem(
+    FilesPipelineTestCaseFieldsMixin, unittest.TestCase
+):
     item_class = FilesPipelineTestAttrsItem
 
 
@@ -262,12 +320,12 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
     default_cls_settings = {
         "EXPIRES": 90,
         "FILES_URLS_FIELD": "file_urls",
-        "FILES_RESULT_FIELD": "files"
+        "FILES_RESULT_FIELD": "files",
     }
     file_cls_attr_settings_map = {
         ("EXPIRES", "FILES_EXPIRES", "expires"),
         ("FILES_URLS_FIELD", "FILES_URLS_FIELD", "files_urls_field"),
-        ("FILES_RESULT_FIELD", "FILES_RESULT_FIELD", "files_result_field")
+        ("FILES_RESULT_FIELD", "FILES_RESULT_FIELD", "files_result_field"),
     }
 
     def setUp(self):
@@ -277,7 +335,6 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         rmtree(self.tempdir)
 
     def _generate_fake_settings(self, prefix=None):
-
         def random_string():
             return "".join([chr(random.randint(97, 123)) for _ in range(10)])
 
@@ -285,15 +342,17 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
             "FILES_EXPIRES": random.randint(100, 1000),
             "FILES_URLS_FIELD": random_string(),
             "FILES_RESULT_FIELD": random_string(),
-            "FILES_STORE": self.tempdir
+            "FILES_STORE": self.tempdir,
         }
         if not prefix:
             return settings
 
-        return {prefix.upper() + "_" + k if k != "FILES_STORE" else k: v for k, v in settings.items()}
+        return {
+            prefix.upper() + "_" + k if k != "FILES_STORE" else k: v
+            for k, v in settings.items()
+        }
 
     def _generate_fake_pipeline(self):
-
         class UserDefinedFilePipeline(FilesPipeline):
             EXPIRES = 1001
             FILES_URLS_FIELD = "alfa"
@@ -346,10 +405,13 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         If there are no settings for subclass and no subclass attributes, pipeline should use
         attributes of base class.
         """
+
         class UserDefinedFilesPipeline(FilesPipeline):
             pass
 
-        user_pipeline = UserDefinedFilesPipeline.from_settings(Settings({"FILES_STORE": self.tempdir}))
+        user_pipeline = UserDefinedFilesPipeline.from_settings(
+            Settings({"FILES_STORE": self.tempdir})
+        )
         for pipe_attr, settings_attr, pipe_ins_attr in self.file_cls_attr_settings_map:
             # Values from settings for custom pipeline should be set on pipeline instance.
             custom_value = self.default_cls_settings.get(pipe_attr.upper())
@@ -360,6 +422,7 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         If there are custom settings for subclass and NO class attributes, pipeline should use custom
         settings.
         """
+
         class UserDefinedFilesPipeline(FilesPipeline):
             pass
 
@@ -381,7 +444,11 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
         prefix = pipeline_cls.__name__.upper()
         settings = self._generate_fake_settings(prefix=prefix)
         user_pipeline = pipeline_cls.from_settings(Settings(settings))
-        for pipe_cls_attr, settings_attr, pipe_inst_attr in self.file_cls_attr_settings_map:
+        for (
+            pipe_cls_attr,
+            settings_attr,
+            pipe_inst_attr,
+        ) in self.file_cls_attr_settings_map:
             custom_value = settings.get(prefix + "_" + settings_attr)
             self.assertNotEqual(custom_value, self.default_cls_settings[pipe_cls_attr])
             self.assertEqual(getattr(user_pipeline, pipe_inst_attr), custom_value)
@@ -391,7 +458,9 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
             DEFAULT_FILES_RESULT_FIELD = "this"
             DEFAULT_FILES_URLS_FIELD = "that"
 
-        pipeline = UserDefinedFilesPipeline.from_settings(Settings({"FILES_STORE": self.tempdir}))
+        pipeline = UserDefinedFilesPipeline.from_settings(
+            Settings({"FILES_STORE": self.tempdir})
+        )
         self.assertEqual(pipeline.files_result_field, "this")
         self.assertEqual(pipeline.files_urls_field, "that")
 
@@ -409,37 +478,36 @@ class FilesPipelineTestCaseCustomSettings(unittest.TestCase):
 
         for pipe_attr, settings_attr, pipe_inst_attr in self.file_cls_attr_settings_map:
             expected_value = settings.get(settings_attr)
-            self.assertEqual(getattr(pipeline_cls, pipe_inst_attr),
-                             expected_value)
+            self.assertEqual(getattr(pipeline_cls, pipe_inst_attr), expected_value)
 
 
 class TestS3FilesStore(unittest.TestCase):
-
     @defer.inlineCallbacks
     def test_persist(self):
         skip_if_no_boto()
 
-        bucket = 'mybucket'
-        key = 'export.csv'
-        uri = f's3://{bucket}/{key}'
+        bucket = "mybucket"
+        key = "export.csv"
+        uri = f"s3://{bucket}/{key}"
         buffer = mock.MagicMock()
-        meta = {'foo': 'bar'}
-        path = ''
-        content_type = 'image/png'
+        meta = {"foo": "bar"}
+        path = ""
+        content_type = "image/png"
 
         store = S3FilesStore(uri)
         from botocore.stub import Stubber
+
         with Stubber(store.s3_client) as stub:
             stub.add_response(
-                'put_object',
+                "put_object",
                 expected_params={
-                    'ACL': S3FilesStore.POLICY,
-                    'Body': buffer,
-                    'Bucket': bucket,
-                    'CacheControl': S3FilesStore.HEADERS['Cache-Control'],
-                    'ContentType': content_type,
-                    'Key': key,
-                    'Metadata': meta,
+                    "ACL": S3FilesStore.POLICY,
+                    "Body": buffer,
+                    "Bucket": bucket,
+                    "CacheControl": S3FilesStore.HEADERS["Cache-Control"],
+                    "ContentType": content_type,
+                    "Key": key,
+                    "Metadata": meta,
                 },
                 service_response={},
             )
@@ -449,7 +517,7 @@ class TestS3FilesStore(unittest.TestCase):
                 buffer,
                 info=None,
                 meta=meta,
-                headers={'Content-Type': content_type},
+                headers={"Content-Type": content_type},
             )
 
             stub.assert_no_pending_responses()
@@ -458,40 +526,41 @@ class TestS3FilesStore(unittest.TestCase):
                 [
                     mock.call.seek(0),
                     # The call to read does not happen with Stubber
-                ]
+                ],
             )
 
     @defer.inlineCallbacks
     def test_stat(self):
         skip_if_no_boto()
 
-        bucket = 'mybucket'
-        key = 'export.csv'
-        uri = f's3://{bucket}/{key}'
-        checksum = '3187896a9657a28163abb31667df64c8'
+        bucket = "mybucket"
+        key = "export.csv"
+        uri = f"s3://{bucket}/{key}"
+        checksum = "3187896a9657a28163abb31667df64c8"
         last_modified = datetime(2019, 12, 1)
 
         store = S3FilesStore(uri)
         from botocore.stub import Stubber
+
         with Stubber(store.s3_client) as stub:
             stub.add_response(
-                'head_object',
+                "head_object",
                 expected_params={
-                    'Bucket': bucket,
-                    'Key': key,
+                    "Bucket": bucket,
+                    "Key": key,
                 },
                 service_response={
-                    'ETag': f'"{checksum}"',
-                    'LastModified': last_modified,
+                    "ETag": f'"{checksum}"',
+                    "LastModified": last_modified,
                 },
             )
 
-            file_stats = yield store.stat_file('', info=None)
+            file_stats = yield store.stat_file("", info=None)
             self.assertEqual(
                 file_stats,
                 {
-                    'checksum': checksum,
-                    'last_modified': last_modified.timestamp(),
+                    "checksum": checksum,
+                    "last_modified": last_modified.timestamp(),
                 },
             )
 
@@ -502,52 +571,82 @@ class TestGCSFilesStore(unittest.TestCase):
     @defer.inlineCallbacks
     def test_persist(self):
         assert_gcs_environ()
-        uri = os.environ.get('GCS_TEST_FILE_URI')
+        uri = os.environ.get("GCS_TEST_FILE_URI")
         if not uri:
             raise unittest.SkipTest("No GCS URI available for testing")
         data = b"TestGCSFilesStore: \xe2\x98\x83"
         buf = BytesIO(data)
-        meta = {'foo': 'bar'}
-        path = 'full/filename'
+        meta = {"foo": "bar"}
+        path = "full/filename"
         store = GCSFilesStore(uri)
-        store.POLICY = 'authenticatedRead'
-        expected_policy = {'role': 'READER', 'entity': 'allAuthenticatedUsers'}
+        store.POLICY = "authenticatedRead"
+        expected_policy = {"role": "READER", "entity": "allAuthenticatedUsers"}
         yield store.persist_file(path, buf, info=None, meta=meta, headers=None)
         s = yield store.stat_file(path, info=None)
-        self.assertIn('last_modified', s)
-        self.assertIn('checksum', s)
-        self.assertEqual(s['checksum'], 'zc2oVgXkbQr2EQdSdw3OPA==')
+        self.assertIn("last_modified", s)
+        self.assertIn("checksum", s)
+        self.assertEqual(s["checksum"], "zc2oVgXkbQr2EQdSdw3OPA==")
         u = urlparse(uri)
         content, acl, blob = get_gcs_content_and_delete(u.hostname, u.path[1:] + path)
         self.assertEqual(content, data)
-        self.assertEqual(blob.metadata, {'foo': 'bar'})
+        self.assertEqual(blob.metadata, {"foo": "bar"})
         self.assertEqual(blob.cache_control, GCSFilesStore.CACHE_CONTROL)
-        self.assertEqual(blob.content_type, 'application/octet-stream')
+        self.assertEqual(blob.content_type, "application/octet-stream")
         self.assertIn(expected_policy, acl)
 
+    @defer.inlineCallbacks
+    def test_blob_path_consistency(self):
+        """Test to make sure that paths used to store files is the same as the one used to get
+        already uploaded files.
+        """
+        assert_gcs_environ()
+        try:
+            import google.cloud.storage  # noqa
+        except ModuleNotFoundError:
+            raise unittest.SkipTest("google-cloud-storage is not installed")
+        else:
+            with mock.patch("google.cloud.storage") as _:
+                with mock.patch("scrapy.pipelines.files.time") as _:
+                    uri = "gs://my_bucket/my_prefix/"
+                    store = GCSFilesStore(uri)
+                    store.bucket = mock.Mock()
+                    path = "full/my_data.txt"
+                    yield store.persist_file(
+                        path, mock.Mock(), info=None, meta=None, headers=None
+                    )
+                    yield store.stat_file(path, info=None)
+                    expected_blob_path = store.prefix + path
+                    store.bucket.blob.assert_called_with(expected_blob_path)
+                    store.bucket.get_blob.assert_called_with(expected_blob_path)
+
 
 class TestFTPFileStore(unittest.TestCase):
     @defer.inlineCallbacks
     def test_persist(self):
-        uri = os.environ.get('FTP_TEST_FILE_URI')
+        uri = os.environ.get("FTP_TEST_FILE_URI")
         if not uri:
             raise unittest.SkipTest("No FTP URI available for testing")
         data = b"TestFTPFilesStore: \xe2\x98\x83"
         buf = BytesIO(data)
-        meta = {'foo': 'bar'}
-        path = 'full/filename'
+        meta = {"foo": "bar"}
+        path = "full/filename"
         store = FTPFilesStore(uri)
         empty_dict = yield store.stat_file(path, info=None)
         self.assertEqual(empty_dict, {})
         yield store.persist_file(path, buf, info=None, meta=meta, headers=None)
         stat = yield store.stat_file(path, info=None)
-        self.assertIn('last_modified', stat)
-        self.assertIn('checksum', stat)
-        self.assertEqual(stat['checksum'], 'd113d66b2ec7258724a268bd88eef6b6')
-        path = f'{store.basedir}/{path}'
+        self.assertIn("last_modified", stat)
+        self.assertIn("checksum", stat)
+        self.assertEqual(stat["checksum"], "d113d66b2ec7258724a268bd88eef6b6")
+        path = f"{store.basedir}/{path}"
         content = get_ftp_content_and_delete(
-            path, store.host, store.port,
-            store.username, store.password, store.USE_ACTIVE_MODE)
+            path,
+            store.host,
+            store.port,
+            store.username,
+            store.password,
+            store.USE_ACTIVE_MODE,
+        )
         self.assertEqual(data.decode(), content)
 
 
@@ -558,14 +657,15 @@ class ItemWithFiles(Item):
 
 def _create_item_with_files(*files):
     item = ItemWithFiles()
-    item['file_urls'] = files
+    item["file_urls"] = files
     return item
 
 
 def _prepare_request_object(item_url, flags=None):
     return Request(
         item_url,
-        meta={'response': Response(item_url, status=200, body=b'data', flags=flags)})
+        meta={"response": Response(item_url, status=200, body=b"data", flags=flags)},
+    )
 
 
 if __name__ == "__main__":
