@@ -1,35 +1,38 @@
-from typing import Optional
 import io
+from typing import Optional
 
 from testfixtures import LogCapture
-from twisted.trial import unittest
-from twisted.python.failure import Failure
 from twisted.internet import reactor
 from twisted.internet.defer import Deferred, inlineCallbacks
+from twisted.python.failure import Failure
+from twisted.trial import unittest
 
 from scrapy import signals
 from scrapy.http import Request, Response
-from scrapy.settings import Settings
-from scrapy.spiders import Spider
+from scrapy.http.request import NO_CALLBACK
 from scrapy.pipelines.files import FileException
 from scrapy.pipelines.images import ImagesPipeline
 from scrapy.pipelines.media import MediaPipeline
+from scrapy.settings import Settings
+from scrapy.spiders import Spider
 from scrapy.utils.deprecate import ScrapyDeprecationWarning
 from scrapy.utils.log import failure_to_exc_info
 from scrapy.utils.signal import disconnect_all
 from scrapy.utils.test import get_crawler
 
-
 try:
     from PIL import Image  # noqa: imported just to check for the import error
 except ImportError:
-    skip_pillow: Optional[str] = 'Missing Python Imaging Library, install https://pypi.python.org/pypi/Pillow'
+    skip_pillow: Optional[
+        str
+    ] = "Missing Python Imaging Library, install https://pypi.python.org/pypi/Pillow"
 else:
     skip_pillow = None
 
 
 def _mocked_download_func(request, info):
-    response = request.meta.get('response')
+    assert request.callback is NO_CALLBACK
+    response = request.meta.get("response")
     return response() if callable(response) else response
 
 
@@ -40,7 +43,7 @@ class BaseMediaPipelineTestCase(unittest.TestCase):
 
     def setUp(self):
         spider_cls = Spider
-        self.spider = spider_cls('media.com')
+        self.spider = spider_cls("media.com")
         crawler = get_crawler(spider_cls, self.settings)
         self.pipe = self.pipeline_class.from_crawler(crawler)
         self.pipe.download_func = _mocked_download_func
@@ -50,29 +53,29 @@ class BaseMediaPipelineTestCase(unittest.TestCase):
 
     def tearDown(self):
         for name, signal in vars(signals).items():
-            if not name.startswith('_'):
+            if not name.startswith("_"):
                 disconnect_all(signal)
 
     def test_default_media_to_download(self):
-        request = Request('http://url')
+        request = Request("http://url")
         assert self.pipe.media_to_download(request, self.info) is None
 
     def test_default_get_media_requests(self):
-        item = dict(name='name')
+        item = dict(name="name")
         assert self.pipe.get_media_requests(item, self.info) is None
 
     def test_default_media_downloaded(self):
-        request = Request('http://url')
-        response = Response('http://url', body=b'')
+        request = Request("http://url")
+        response = Response("http://url", body=b"")
         assert self.pipe.media_downloaded(response, request, self.info) is response
 
     def test_default_media_failed(self):
-        request = Request('http://url')
+        request = Request("http://url")
         fail = Failure(Exception())
         assert self.pipe.media_failed(fail, request, self.info) is fail
 
     def test_default_item_completed(self):
-        item = dict(name='name')
+        item = dict(name="name")
         assert self.pipe.item_completed([], item, self.info) is item
 
         # Check that failures are logged by default
@@ -85,7 +88,7 @@ class BaseMediaPipelineTestCase(unittest.TestCase):
         assert new_item is item
         assert len(log.records) == 1
         record = log.records[0]
-        assert record.levelname == 'ERROR'
+        assert record.levelname == "ERROR"
         self.assertTupleEqual(record.exc_info, failure_to_exc_info(fail))
 
         # disable failure logging and check again
@@ -97,14 +100,14 @@ class BaseMediaPipelineTestCase(unittest.TestCase):
 
     @inlineCallbacks
     def test_default_process_item(self):
-        item = dict(name='name')
+        item = dict(name="name")
         new_item = yield self.pipe.process_item(item, self.spider)
         assert new_item is item
 
     def test_modify_media_request(self):
-        request = Request('http://url')
+        request = Request("http://url")
         self.pipe._modify_media_request(request)
-        assert request.meta == {'handle_httpstatus_all': True}
+        assert request.meta == {"handle_httpstatus_all": True}
 
     def test_should_remove_req_res_references_before_caching_the_results(self):
         """Regression test case to prevent a memory leak in the Media Pipeline.
@@ -134,8 +137,8 @@ class BaseMediaPipelineTestCase(unittest.TestCase):
         Chaining (https://www.python.org/dev/peps/pep-3134/).
         """
         # Create sample pair of Request and Response objects
-        request = Request('http://url')
-        response = Response('http://url', body=b'', request=request)
+        request = Request("http://url")
+        response = Response("http://url", body=b"", request=request)
 
         # Simulate the Media Pipeline behavior to produce a Twisted Failure
         try:
@@ -146,7 +149,7 @@ class BaseMediaPipelineTestCase(unittest.TestCase):
             try:
                 # Simulate the media_downloaded callback raising a FileException
                 # This usually happens when the status code is not 200 OK
-                raise FileException('download-error')
+                raise FileException("download-error")
             except Exception as exc:
                 file_exc = exc
                 # Simulate Twisted capturing the FileException
@@ -171,42 +174,41 @@ class BaseMediaPipelineTestCase(unittest.TestCase):
         # ... encapsulating the original FileException ...
         self.assertEqual(info.downloaded[fp].value, file_exc)
         # ... but it should not store the StopIteration exception on its context
-        context = getattr(info.downloaded[fp].value, '__context__', None)
+        context = getattr(info.downloaded[fp].value, "__context__", None)
         self.assertIsNone(context)
 
 
 class MockedMediaPipeline(MediaPipeline):
-
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._mockcalled = []
 
     def download(self, request, info):
-        self._mockcalled.append('download')
+        self._mockcalled.append("download")
         return super().download(request, info)
 
     def media_to_download(self, request, info, *, item=None):
-        self._mockcalled.append('media_to_download')
-        if 'result' in request.meta:
-            return request.meta.get('result')
+        self._mockcalled.append("media_to_download")
+        if "result" in request.meta:
+            return request.meta.get("result")
         return super().media_to_download(request, info)
 
     def get_media_requests(self, item, info):
-        self._mockcalled.append('get_media_requests')
-        return item.get('requests')
+        self._mockcalled.append("get_media_requests")
+        return item.get("requests")
 
     def media_downloaded(self, response, request, info, *, item=None):
-        self._mockcalled.append('media_downloaded')
+        self._mockcalled.append("media_downloaded")
         return super().media_downloaded(response, request, info)
 
     def media_failed(self, failure, request, info):
-        self._mockcalled.append('media_failed')
+        self._mockcalled.append("media_failed")
         return super().media_failed(failure, request, info)
 
     def item_completed(self, results, item, info):
-        self._mockcalled.append('item_completed')
+        self._mockcalled.append("item_completed")
         item = super().item_completed(results, item, info)
-        item['results'] = results
+        item["results"] = results
         return item
 
 
@@ -215,72 +217,94 @@ class MediaPipelineTestCase(BaseMediaPipelineTestCase):
     pipeline_class = MockedMediaPipeline
 
     def _callback(self, result):
-        self.pipe._mockcalled.append('request_callback')
+        self.pipe._mockcalled.append("request_callback")
         return result
 
     def _errback(self, result):
-        self.pipe._mockcalled.append('request_errback')
+        self.pipe._mockcalled.append("request_errback")
         return result
 
     @inlineCallbacks
     def test_result_succeed(self):
-        rsp = Response('http://url1')
-        req = Request('http://url1', meta=dict(response=rsp),
-                      callback=self._callback, errback=self._errback)
+        rsp = Response("http://url1")
+        req = Request(
+            "http://url1",
+            meta=dict(response=rsp),
+            callback=self._callback,
+            errback=self._errback,
+        )
         item = dict(requests=req)
         new_item = yield self.pipe.process_item(item, self.spider)
-        self.assertEqual(new_item['results'], [(True, rsp)])
+        self.assertEqual(new_item["results"], [(True, rsp)])
         self.assertEqual(
             self.pipe._mockcalled,
-            ['get_media_requests', 'media_to_download', 'media_downloaded', 'request_callback', 'item_completed'])
+            [
+                "get_media_requests",
+                "media_to_download",
+                "media_downloaded",
+                "request_callback",
+                "item_completed",
+            ],
+        )
 
     @inlineCallbacks
     def test_result_failure(self):
         self.pipe.LOG_FAILED_RESULTS = False
         fail = Failure(Exception())
-        req = Request('http://url1', meta=dict(response=fail),
-                      callback=self._callback, errback=self._errback)
+        req = Request(
+            "http://url1",
+            meta=dict(response=fail),
+            callback=self._callback,
+            errback=self._errback,
+        )
         item = dict(requests=req)
         new_item = yield self.pipe.process_item(item, self.spider)
-        self.assertEqual(new_item['results'], [(False, fail)])
+        self.assertEqual(new_item["results"], [(False, fail)])
         self.assertEqual(
             self.pipe._mockcalled,
-            ['get_media_requests', 'media_to_download', 'media_failed', 'request_errback', 'item_completed'])
+            [
+                "get_media_requests",
+                "media_to_download",
+                "media_failed",
+                "request_errback",
+                "item_completed",
+            ],
+        )
 
     @inlineCallbacks
     def test_mix_of_success_and_failure(self):
         self.pipe.LOG_FAILED_RESULTS = False
-        rsp1 = Response('http://url1')
-        req1 = Request('http://url1', meta=dict(response=rsp1))
+        rsp1 = Response("http://url1")
+        req1 = Request("http://url1", meta=dict(response=rsp1))
         fail = Failure(Exception())
-        req2 = Request('http://url2', meta=dict(response=fail))
+        req2 = Request("http://url2", meta=dict(response=fail))
         item = dict(requests=[req1, req2])
         new_item = yield self.pipe.process_item(item, self.spider)
-        self.assertEqual(new_item['results'], [(True, rsp1), (False, fail)])
+        self.assertEqual(new_item["results"], [(True, rsp1), (False, fail)])
         m = self.pipe._mockcalled
         # only once
-        self.assertEqual(m[0], 'get_media_requests')  # first hook called
-        self.assertEqual(m.count('get_media_requests'), 1)
-        self.assertEqual(m.count('item_completed'), 1)
-        self.assertEqual(m[-1], 'item_completed')  # last hook called
+        self.assertEqual(m[0], "get_media_requests")  # first hook called
+        self.assertEqual(m.count("get_media_requests"), 1)
+        self.assertEqual(m.count("item_completed"), 1)
+        self.assertEqual(m[-1], "item_completed")  # last hook called
         # twice, one per request
-        self.assertEqual(m.count('media_to_download'), 2)
+        self.assertEqual(m.count("media_to_download"), 2)
         # one to handle success and other for failure
-        self.assertEqual(m.count('media_downloaded'), 1)
-        self.assertEqual(m.count('media_failed'), 1)
+        self.assertEqual(m.count("media_downloaded"), 1)
+        self.assertEqual(m.count("media_failed"), 1)
 
     @inlineCallbacks
     def test_get_media_requests(self):
         # returns single Request (without callback)
-        req = Request('http://url')
+        req = Request("http://url")
         item = dict(requests=req)  # pass a single item
         new_item = yield self.pipe.process_item(item, self.spider)
         assert new_item is item
         self.assertIn(self.fingerprint(req), self.info.downloaded)
 
         # returns iterable of Requests
-        req1 = Request('http://url1')
-        req2 = Request('http://url2')
+        req1 = Request("http://url1")
+        req2 = Request("http://url2")
         item = dict(requests=iter([req1, req2]))
         new_item = yield self.pipe.process_item(item, self.spider)
         assert new_item is item
@@ -289,30 +313,34 @@ class MediaPipelineTestCase(BaseMediaPipelineTestCase):
 
     @inlineCallbacks
     def test_results_are_cached_across_multiple_items(self):
-        rsp1 = Response('http://url1')
-        req1 = Request('http://url1', meta=dict(response=rsp1))
+        rsp1 = Response("http://url1")
+        req1 = Request("http://url1", meta=dict(response=rsp1))
         item = dict(requests=req1)
         new_item = yield self.pipe.process_item(item, self.spider)
         self.assertTrue(new_item is item)
-        self.assertEqual(new_item['results'], [(True, rsp1)])
+        self.assertEqual(new_item["results"], [(True, rsp1)])
 
         # rsp2 is ignored, rsp1 must be in results because request fingerprints are the same
-        req2 = Request(req1.url, meta=dict(response=Response('http://donot.download.me')))
+        req2 = Request(
+            req1.url, meta=dict(response=Response("http://donot.download.me"))
+        )
         item = dict(requests=req2)
         new_item = yield self.pipe.process_item(item, self.spider)
         self.assertTrue(new_item is item)
         self.assertEqual(self.fingerprint(req1), self.fingerprint(req2))
-        self.assertEqual(new_item['results'], [(True, rsp1)])
+        self.assertEqual(new_item["results"], [(True, rsp1)])
 
     @inlineCallbacks
     def test_results_are_cached_for_requests_of_single_item(self):
-        rsp1 = Response('http://url1')
-        req1 = Request('http://url1', meta=dict(response=rsp1))
-        req2 = Request(req1.url, meta=dict(response=Response('http://donot.download.me')))
+        rsp1 = Response("http://url1")
+        req1 = Request("http://url1", meta=dict(response=rsp1))
+        req2 = Request(
+            req1.url, meta=dict(response=Response("http://donot.download.me"))
+        )
         item = dict(requests=[req1, req2])
         new_item = yield self.pipe.process_item(item, self.spider)
         self.assertTrue(new_item is item)
-        self.assertEqual(new_item['results'], [(True, rsp1), (True, rsp1)])
+        self.assertEqual(new_item["results"], [(True, rsp1), (True, rsp1)])
 
     @inlineCallbacks
     def test_wait_if_request_is_downloading(self):
@@ -324,78 +352,80 @@ class MediaPipelineTestCase(BaseMediaPipelineTestCase):
             self.assertEqual(len(self.info.waiting[fp]), 2)
             return response
 
-        rsp1 = Response('http://url')
+        rsp1 = Response("http://url")
 
         def rsp1_func():
             dfd = Deferred().addCallback(_check_downloading)
-            reactor.callLater(.1, dfd.callback, rsp1)
+            reactor.callLater(0.1, dfd.callback, rsp1)
             return dfd
 
         def rsp2_func():
-            self.fail('it must cache rsp1 result and must not try to redownload')
+            self.fail("it must cache rsp1 result and must not try to redownload")
 
-        req1 = Request('http://url', meta=dict(response=rsp1_func))
+        req1 = Request("http://url", meta=dict(response=rsp1_func))
         req2 = Request(req1.url, meta=dict(response=rsp2_func))
         item = dict(requests=[req1, req2])
         new_item = yield self.pipe.process_item(item, self.spider)
-        self.assertEqual(new_item['results'], [(True, rsp1), (True, rsp1)])
+        self.assertEqual(new_item["results"], [(True, rsp1), (True, rsp1)])
 
     @inlineCallbacks
     def test_use_media_to_download_result(self):
-        req = Request('http://url', meta=dict(result='ITSME', response=self.fail))
+        req = Request("http://url", meta=dict(result="ITSME", response=self.fail))
         item = dict(requests=req)
         new_item = yield self.pipe.process_item(item, self.spider)
-        self.assertEqual(new_item['results'], [(True, 'ITSME')])
+        self.assertEqual(new_item["results"], [(True, "ITSME")])
         self.assertEqual(
             self.pipe._mockcalled,
-            ['get_media_requests', 'media_to_download', 'item_completed'])
+            ["get_media_requests", "media_to_download", "item_completed"],
+        )
 
 
 class MockedMediaPipelineDeprecatedMethods(ImagesPipeline):
-
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._mockcalled = []
 
     def get_media_requests(self, item, info):
-        item_url = item['image_urls'][0]
+        item_url = item["image_urls"][0]
         output_img = io.BytesIO()
-        img = Image.new('RGB', (60, 30), color='red')
-        img.save(output_img, format='JPEG')
+        img = Image.new("RGB", (60, 30), color="red")
+        img.save(output_img, format="JPEG")
         return Request(
             item_url,
-            meta={'response': Response(item_url, status=200, body=output_img.getvalue())}
+            meta={
+                "response": Response(item_url, status=200, body=output_img.getvalue())
+            },
         )
 
     def inc_stats(self, *args, **kwargs):
         return True
 
     def media_to_download(self, request, info):
-        self._mockcalled.append('media_to_download')
+        self._mockcalled.append("media_to_download")
         return super().media_to_download(request, info)
 
     def media_downloaded(self, response, request, info):
-        self._mockcalled.append('media_downloaded')
+        self._mockcalled.append("media_downloaded")
         return super().media_downloaded(response, request, info)
 
     def file_downloaded(self, response, request, info):
-        self._mockcalled.append('file_downloaded')
+        self._mockcalled.append("file_downloaded")
         return super().file_downloaded(response, request, info)
 
     def file_path(self, request, response=None, info=None):
-        self._mockcalled.append('file_path')
+        self._mockcalled.append("file_path")
         return super().file_path(request, response, info)
 
     def thumb_path(self, request, thumb_id, response=None, info=None):
-        self._mockcalled.append('thumb_path')
-        return super(MockedMediaPipelineDeprecatedMethods, self).thumb_path(request, thumb_id, response, info)
+        self._mockcalled.append("thumb_path")
+        return super().thumb_path(request, thumb_id, response, info)
 
     def get_images(self, response, request, info):
-        self._mockcalled.append('get_images')
-        return super(MockedMediaPipelineDeprecatedMethods, self).get_images(response, request, info)
+        self._mockcalled.append("get_images")
+        return super().get_images(response, request, info)
 
     def image_downloaded(self, response, request, info):
-        self._mockcalled.append('image_downloaded')
+        self._mockcalled.append("image_downloaded")
         return super().image_downloaded(response, request, info)
 
 
@@ -404,20 +434,23 @@ class MediaPipelineDeprecatedMethodsTestCase(unittest.TestCase):
 
     def setUp(self):
         settings_dict = {
-            'IMAGES_STORE': 'store-uri',
-            'IMAGES_THUMBS': {'small': (50, 50)},
+            "IMAGES_STORE": "store-uri",
+            "IMAGES_THUMBS": {"small": (50, 50)},
         }
         crawler = get_crawler(spidercls=None, settings_dict=settings_dict)
         self.pipe = MockedMediaPipelineDeprecatedMethods.from_crawler(crawler)
         self.pipe.download_func = _mocked_download_func
         self.pipe.open_spider(None)
-        self.item = dict(image_urls=['http://picsum.photos/id/1014/200/300'], images=[])
+        self.item = dict(image_urls=["http://picsum.photos/id/1014/200/300"], images=[])
 
     def _assert_method_called_with_warnings(self, method, message, warnings):
         self.assertIn(method, self.pipe._mockcalled)
         warningShown = False
         for warning in warnings:
-            if warning['message'] == message and warning['category'] == ScrapyDeprecationWarning:
+            if (
+                warning["message"] == message
+                and warning["category"] == ScrapyDeprecationWarning
+            ):
                 warningShown = True
         self.assertTrue(warningShown)
 
@@ -426,125 +459,111 @@ class MediaPipelineDeprecatedMethodsTestCase(unittest.TestCase):
         yield self.pipe.process_item(self.item, None)
         warnings = self.flushWarnings([MediaPipeline._compatible])
         message = (
-            'media_to_download(self, request, info) is deprecated, '
-            'please use media_to_download(self, request, info, *, item=None)'
+            "media_to_download(self, request, info) is deprecated, "
+            "please use media_to_download(self, request, info, *, item=None)"
         )
-        self._assert_method_called_with_warnings('media_to_download', message, warnings)
+        self._assert_method_called_with_warnings("media_to_download", message, warnings)
 
     @inlineCallbacks
     def test_media_downloaded_called(self):
         yield self.pipe.process_item(self.item, None)
         warnings = self.flushWarnings([MediaPipeline._compatible])
         message = (
-            'media_downloaded(self, response, request, info) is deprecated, '
-            'please use media_downloaded(self, response, request, info, *, item=None)'
+            "media_downloaded(self, response, request, info) is deprecated, "
+            "please use media_downloaded(self, response, request, info, *, item=None)"
         )
-        self._assert_method_called_with_warnings('media_downloaded', message, warnings)
+        self._assert_method_called_with_warnings("media_downloaded", message, warnings)
 
     @inlineCallbacks
     def test_file_downloaded_called(self):
         yield self.pipe.process_item(self.item, None)
         warnings = self.flushWarnings([MediaPipeline._compatible])
         message = (
-            'file_downloaded(self, response, request, info) is deprecated, '
-            'please use file_downloaded(self, response, request, info, *, item=None)'
+            "file_downloaded(self, response, request, info) is deprecated, "
+            "please use file_downloaded(self, response, request, info, *, item=None)"
         )
-        self._assert_method_called_with_warnings('file_downloaded', message, warnings)
+        self._assert_method_called_with_warnings("file_downloaded", message, warnings)
 
     @inlineCallbacks
     def test_file_path_called(self):
         yield self.pipe.process_item(self.item, None)
         warnings = self.flushWarnings([MediaPipeline._compatible])
         message = (
-            'file_path(self, request, response=None, info=None) is deprecated, '
-            'please use file_path(self, request, response=None, info=None, *, item=None)'
+            "file_path(self, request, response=None, info=None) is deprecated, "
+            "please use file_path(self, request, response=None, info=None, *, item=None)"
         )
-        self._assert_method_called_with_warnings('file_path', message, warnings)
+        self._assert_method_called_with_warnings("file_path", message, warnings)
 
     @inlineCallbacks
     def test_thumb_path_called(self):
         yield self.pipe.process_item(self.item, None)
         warnings = self.flushWarnings([MediaPipeline._compatible])
         message = (
-            'thumb_path(self, request, thumb_id, response=None, info=None) is deprecated, '
-            'please use thumb_path(self, request, thumb_id, response=None, info=None, *, item=None)'
+            "thumb_path(self, request, thumb_id, response=None, info=None) is deprecated, "
+            "please use thumb_path(self, request, thumb_id, response=None, info=None, *, item=None)"
         )
-        self._assert_method_called_with_warnings('thumb_path', message, warnings)
+        self._assert_method_called_with_warnings("thumb_path", message, warnings)
 
     @inlineCallbacks
     def test_get_images_called(self):
         yield self.pipe.process_item(self.item, None)
         warnings = self.flushWarnings([MediaPipeline._compatible])
         message = (
-            'get_images(self, response, request, info) is deprecated, '
-            'please use get_images(self, response, request, info, *, item=None)'
+            "get_images(self, response, request, info) is deprecated, "
+            "please use get_images(self, response, request, info, *, item=None)"
         )
-        self._assert_method_called_with_warnings('get_images', message, warnings)
+        self._assert_method_called_with_warnings("get_images", message, warnings)
 
     @inlineCallbacks
     def test_image_downloaded_called(self):
         yield self.pipe.process_item(self.item, None)
         warnings = self.flushWarnings([MediaPipeline._compatible])
         message = (
-            'image_downloaded(self, response, request, info) is deprecated, '
-            'please use image_downloaded(self, response, request, info, *, item=None)'
+            "image_downloaded(self, response, request, info) is deprecated, "
+            "please use image_downloaded(self, response, request, info, *, item=None)"
         )
-        self._assert_method_called_with_warnings('image_downloaded', message, warnings)
+        self._assert_method_called_with_warnings("image_downloaded", message, warnings)
 
 
 class MediaPipelineAllowRedirectSettingsTestCase(unittest.TestCase):
-
     def _assert_request_no3xx(self, pipeline_class, settings):
         pipe = pipeline_class(settings=Settings(settings))
-        request = Request('http://url')
+        request = Request("http://url")
         pipe._modify_media_request(request)
 
-        self.assertIn('handle_httpstatus_list', request.meta)
+        self.assertIn("handle_httpstatus_list", request.meta)
         for status, check in [
-                (200, True),
-
-                # These are the status codes we want
-                # the downloader to handle itself
-                (301, False),
-                (302, False),
-                (302, False),
-                (307, False),
-                (308, False),
-
-                # we still want to get 4xx and 5xx
-                (400, True),
-                (404, True),
-                (500, True)]:
+            (200, True),
+            # These are the status codes we want
+            # the downloader to handle itself
+            (301, False),
+            (302, False),
+            (302, False),
+            (307, False),
+            (308, False),
+            # we still want to get 4xx and 5xx
+            (400, True),
+            (404, True),
+            (500, True),
+        ]:
             if check:
-                self.assertIn(status, request.meta['handle_httpstatus_list'])
+                self.assertIn(status, request.meta["handle_httpstatus_list"])
             else:
-                self.assertNotIn(status, request.meta['handle_httpstatus_list'])
+                self.assertNotIn(status, request.meta["handle_httpstatus_list"])
 
     def test_standard_setting(self):
-        self._assert_request_no3xx(
-            MediaPipeline,
-            {
-                'MEDIA_ALLOW_REDIRECTS': True
-            })
+        self._assert_request_no3xx(MediaPipeline, {"MEDIA_ALLOW_REDIRECTS": True})
 
     def test_subclass_standard_setting(self):
-
         class UserDefinedPipeline(MediaPipeline):
             pass
 
-        self._assert_request_no3xx(
-            UserDefinedPipeline,
-            {
-                'MEDIA_ALLOW_REDIRECTS': True
-            })
+        self._assert_request_no3xx(UserDefinedPipeline, {"MEDIA_ALLOW_REDIRECTS": True})
 
     def test_subclass_specific_setting(self):
-
         class UserDefinedPipeline(MediaPipeline):
             pass
 
         self._assert_request_no3xx(
-            UserDefinedPipeline,
-            {
-                'USERDEFINEDPIPELINE_MEDIA_ALLOW_REDIRECTS': True
-            })
+            UserDefinedPipeline, {"USERDEFINEDPIPELINE_MEDIA_ALLOW_REDIRECTS": True}
+        )
