@@ -1,33 +1,33 @@
-import os
-from datetime import datetime
 import shutil
+from datetime import datetime
+from pathlib import Path
+
 from twisted.trial import unittest
 
+from scrapy.exceptions import NotConfigured
 from scrapy.extensions.spiderstate import SpiderState
 from scrapy.spiders import Spider
-from scrapy.exceptions import NotConfigured
 from scrapy.utils.test import get_crawler
 
 
 class SpiderStateTest(unittest.TestCase):
-
     def test_store_load(self):
         jobdir = self.mktemp()
-        os.mkdir(jobdir)
+        Path(jobdir).mkdir()
         try:
-            spider = Spider(name='default')
+            spider = Spider(name="default")
             dt = datetime.now()
 
             ss = SpiderState(jobdir)
             ss.spider_opened(spider)
-            spider.state['one'] = 1
-            spider.state['dt'] = dt
+            spider.state["one"] = 1
+            spider.state["dt"] = dt
             ss.spider_closed(spider)
 
-            spider2 = Spider(name='default')
+            spider2 = Spider(name="default")
             ss2 = SpiderState(jobdir)
             ss2.spider_opened(spider2)
-            self.assertEqual(spider.state, {'one': 1, 'dt': dt})
+            self.assertEqual(spider.state, {"one": 1, "dt": dt})
             ss2.spider_closed(spider2)
         finally:
             shutil.rmtree(jobdir)
@@ -35,7 +35,7 @@ class SpiderStateTest(unittest.TestCase):
     def test_state_attribute(self):
         # state attribute must be present if jobdir is not set, to provide a
         # consistent interface
-        spider = Spider(name='default')
+        spider = Spider(name="default")
         ss = SpiderState()
         ss.spider_opened(spider)
         self.assertEqual(spider.state, {})
