@@ -1,7 +1,8 @@
 import inspect
 import unittest
-from unittest import mock
 import warnings
+from unittest import mock
+
 from scrapy.exceptions import ScrapyDeprecationWarning
 from scrapy.utils.deprecate import create_deprecated_class, update_classpath
 
@@ -19,22 +20,23 @@ class NewName(SomeBaseClass):
 
 
 class WarnWhenSubclassedTest(unittest.TestCase):
-
     def _mywarnings(self, w, category=MyWarning):
         return [x for x in w if x.category is MyWarning]
 
     def test_no_warning_on_definition(self):
         with warnings.catch_warnings(record=True) as w:
-            create_deprecated_class('Deprecated', NewName)
+            create_deprecated_class("Deprecated", NewName)
 
         w = self._mywarnings(w)
         self.assertEqual(w, [])
 
     def test_subclassing_warning_message(self):
-        Deprecated = create_deprecated_class('Deprecated', NewName,
-                                             warn_category=MyWarning)
+        Deprecated = create_deprecated_class(
+            "Deprecated", NewName, warn_category=MyWarning
+        )
 
         with warnings.catch_warnings(record=True) as w:
+
             class UserClass(Deprecated):
                 pass
 
@@ -45,17 +47,21 @@ class WarnWhenSubclassedTest(unittest.TestCase):
             "tests.test_utils_deprecate.UserClass inherits from "
             "deprecated class tests.test_utils_deprecate.Deprecated, "
             "please inherit from tests.test_utils_deprecate.NewName."
-            " (warning only on first subclass, there may be others)"
+            " (warning only on first subclass, there may be others)",
         )
         self.assertEqual(w[0].lineno, inspect.getsourcelines(UserClass)[1])
 
     def test_custom_class_paths(self):
-        Deprecated = create_deprecated_class('Deprecated', NewName,
-                                             new_class_path='foo.NewClass',
-                                             old_class_path='bar.OldClass',
-                                             warn_category=MyWarning)
+        Deprecated = create_deprecated_class(
+            "Deprecated",
+            NewName,
+            new_class_path="foo.NewClass",
+            old_class_path="bar.OldClass",
+            warn_category=MyWarning,
+        )
 
         with warnings.catch_warnings(record=True) as w:
+
             class UserClass(Deprecated):
                 pass
 
@@ -63,17 +69,18 @@ class WarnWhenSubclassedTest(unittest.TestCase):
 
         w = self._mywarnings(w)
         self.assertEqual(len(w), 2)
-        self.assertIn('foo.NewClass', str(w[0].message))
-        self.assertIn('bar.OldClass', str(w[0].message))
-        self.assertIn('foo.NewClass', str(w[1].message))
-        self.assertIn('bar.OldClass', str(w[1].message))
+        self.assertIn("foo.NewClass", str(w[0].message))
+        self.assertIn("bar.OldClass", str(w[0].message))
+        self.assertIn("foo.NewClass", str(w[1].message))
+        self.assertIn("bar.OldClass", str(w[1].message))
 
-    def test_subclassing_warns_only_on_direct_childs(self):
-        Deprecated = create_deprecated_class('Deprecated', NewName,
-                                             warn_once=False,
-                                             warn_category=MyWarning)
+    def test_subclassing_warns_only_on_direct_children(self):
+        Deprecated = create_deprecated_class(
+            "Deprecated", NewName, warn_once=False, warn_category=MyWarning
+        )
 
         with warnings.catch_warnings(record=True) as w:
+
             class UserClass(Deprecated):
                 pass
 
@@ -82,13 +89,15 @@ class WarnWhenSubclassedTest(unittest.TestCase):
 
         w = self._mywarnings(w)
         self.assertEqual(len(w), 1)
-        self.assertIn('UserClass', str(w[0].message))
+        self.assertIn("UserClass", str(w[0].message))
 
     def test_subclassing_warns_once_by_default(self):
-        Deprecated = create_deprecated_class('Deprecated', NewName,
-                                             warn_category=MyWarning)
+        Deprecated = create_deprecated_class(
+            "Deprecated", NewName, warn_category=MyWarning
+        )
 
         with warnings.catch_warnings(record=True) as w:
+
             class UserClass(Deprecated):
                 pass
 
@@ -100,15 +109,16 @@ class WarnWhenSubclassedTest(unittest.TestCase):
 
         w = self._mywarnings(w)
         self.assertEqual(len(w), 1)
-        self.assertIn('UserClass', str(w[0].message))
+        self.assertIn("UserClass", str(w[0].message))
 
     def test_warning_on_instance(self):
-        Deprecated = create_deprecated_class('Deprecated', NewName,
-                                             warn_category=MyWarning)
+        Deprecated = create_deprecated_class(
+            "Deprecated", NewName, warn_category=MyWarning
+        )
 
         # ignore subclassing warnings
         with warnings.catch_warnings():
-            warnings.simplefilter('ignore', ScrapyDeprecationWarning)
+            warnings.simplefilter("ignore", MyWarning)
 
             class UserClass(Deprecated):
                 pass
@@ -122,13 +132,13 @@ class WarnWhenSubclassedTest(unittest.TestCase):
         self.assertEqual(
             str(w[0].message),
             "tests.test_utils_deprecate.Deprecated is deprecated, "
-            "instantiate tests.test_utils_deprecate.NewName instead."
+            "instantiate tests.test_utils_deprecate.NewName instead.",
         )
         self.assertEqual(w[0].lineno, lineno)
 
     def test_warning_auto_message(self):
         with warnings.catch_warnings(record=True) as w:
-            Deprecated = create_deprecated_class('Deprecated', NewName)
+            Deprecated = create_deprecated_class("Deprecated", NewName)
 
             class UserClass2(Deprecated):
                 pass
@@ -139,8 +149,8 @@ class WarnWhenSubclassedTest(unittest.TestCase):
 
     def test_issubclass(self):
         with warnings.catch_warnings():
-            warnings.simplefilter('ignore', ScrapyDeprecationWarning)
-            DeprecatedName = create_deprecated_class('DeprecatedName', NewName)
+            warnings.simplefilter("ignore", ScrapyDeprecationWarning)
+            DeprecatedName = create_deprecated_class("DeprecatedName", NewName)
 
             class UpdatedUserClass1(NewName):
                 pass
@@ -175,8 +185,8 @@ class WarnWhenSubclassedTest(unittest.TestCase):
 
     def test_isinstance(self):
         with warnings.catch_warnings():
-            warnings.simplefilter('ignore', ScrapyDeprecationWarning)
-            DeprecatedName = create_deprecated_class('DeprecatedName', NewName)
+            warnings.simplefilter("ignore", ScrapyDeprecationWarning)
+            DeprecatedName = create_deprecated_class("DeprecatedName", NewName)
 
             class UpdatedUserClass2(NewName):
                 pass
@@ -209,24 +219,28 @@ class WarnWhenSubclassedTest(unittest.TestCase):
 
     def test_clsdict(self):
         with warnings.catch_warnings():
-            warnings.simplefilter('ignore', ScrapyDeprecationWarning)
-            Deprecated = create_deprecated_class('Deprecated', NewName, {'foo': 'bar'})
+            warnings.simplefilter("ignore", ScrapyDeprecationWarning)
+            Deprecated = create_deprecated_class("Deprecated", NewName, {"foo": "bar"})
 
-        self.assertEqual(Deprecated.foo, 'bar')
+        self.assertEqual(Deprecated.foo, "bar")
 
     def test_deprecate_a_class_with_custom_metaclass(self):
-        Meta1 = type('Meta1', (type,), {})
-        New = Meta1('New', (), {})
-        create_deprecated_class('Deprecated', New)
+        Meta1 = type("Meta1", (type,), {})
+        New = Meta1("New", (), {})
+        create_deprecated_class("Deprecated", New)
 
     def test_deprecate_subclass_of_deprecated_class(self):
         with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter('always')
-            Deprecated = create_deprecated_class('Deprecated', NewName,
-                                                 warn_category=MyWarning)
-            AlsoDeprecated = create_deprecated_class('AlsoDeprecated', Deprecated,
-                                                     new_class_path='foo.Bar',
-                                                     warn_category=MyWarning)
+            warnings.simplefilter("always")
+            Deprecated = create_deprecated_class(
+                "Deprecated", NewName, warn_category=MyWarning
+            )
+            AlsoDeprecated = create_deprecated_class(
+                "AlsoDeprecated",
+                Deprecated,
+                new_class_path="foo.Bar",
+                warn_category=MyWarning,
+            )
 
         w = self._mywarnings(w)
         self.assertEqual(len(w), 0, str(map(str, w)))
@@ -239,15 +253,15 @@ class WarnWhenSubclassedTest(unittest.TestCase):
 
         w = self._mywarnings(w)
         self.assertEqual(len(w), 2)
-        self.assertIn('AlsoDeprecated', str(w[0].message))
-        self.assertIn('foo.Bar', str(w[0].message))
-        self.assertIn('AlsoDeprecated', str(w[1].message))
-        self.assertIn('foo.Bar', str(w[1].message))
+        self.assertIn("AlsoDeprecated", str(w[0].message))
+        self.assertIn("foo.Bar", str(w[0].message))
+        self.assertIn("AlsoDeprecated", str(w[1].message))
+        self.assertIn("foo.Bar", str(w[1].message))
 
     def test_inspect_stack(self):
-        with mock.patch('inspect.stack', side_effect=IndexError):
+        with mock.patch("inspect.stack", side_effect=IndexError):
             with warnings.catch_warnings(record=True) as w:
-                DeprecatedName = create_deprecated_class('DeprecatedName', NewName)
+                DeprecatedName = create_deprecated_class("DeprecatedName", NewName)
 
                 class SubClass(DeprecatedName):
                     pass
@@ -255,27 +269,30 @@ class WarnWhenSubclassedTest(unittest.TestCase):
         self.assertIn("Error detecting parent module", str(w[0].message))
 
 
-@mock.patch('scrapy.utils.deprecate.DEPRECATION_RULES',
-            [('scrapy.contrib.pipeline.', 'scrapy.pipelines.'),
-             ('scrapy.contrib.', 'scrapy.extensions.')])
+@mock.patch(
+    "scrapy.utils.deprecate.DEPRECATION_RULES",
+    [
+        ("scrapy.contrib.pipeline.", "scrapy.pipelines."),
+        ("scrapy.contrib.", "scrapy.extensions."),
+    ],
+)
 class UpdateClassPathTest(unittest.TestCase):
-
     def test_old_path_gets_fixed(self):
         with warnings.catch_warnings(record=True) as w:
-            output = update_classpath('scrapy.contrib.debug.Debug')
-        self.assertEqual(output, 'scrapy.extensions.debug.Debug')
+            output = update_classpath("scrapy.contrib.debug.Debug")
+        self.assertEqual(output, "scrapy.extensions.debug.Debug")
         self.assertEqual(len(w), 1)
         self.assertIn("scrapy.contrib.debug.Debug", str(w[0].message))
         self.assertIn("scrapy.extensions.debug.Debug", str(w[0].message))
 
     def test_sorted_replacement(self):
         with warnings.catch_warnings():
-            warnings.simplefilter('ignore', ScrapyDeprecationWarning)
-            output = update_classpath('scrapy.contrib.pipeline.Pipeline')
-        self.assertEqual(output, 'scrapy.pipelines.Pipeline')
+            warnings.simplefilter("ignore", ScrapyDeprecationWarning)
+            output = update_classpath("scrapy.contrib.pipeline.Pipeline")
+        self.assertEqual(output, "scrapy.pipelines.Pipeline")
 
     def test_unmatched_path_stays_the_same(self):
         with warnings.catch_warnings(record=True) as w:
-            output = update_classpath('scrapy.unmatched.Path')
-        self.assertEqual(output, 'scrapy.unmatched.Path')
+            output = update_classpath("scrapy.unmatched.Path")
+        self.assertEqual(output, "scrapy.unmatched.Path")
         self.assertEqual(len(w), 0)
